#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Heresh Fattahi, Aug 2022      #
############################################################


import glob
import os
import sys

from mintpy.utils.arg_utils import create_argument_parser

####################################################################################
TEMPLATE = """template options:
  ########## 1. load_data
  ## no   - save   0% disk usage, fast [default]
  ## lzf  - save ~57% disk usage, relative slow
  ## gzip - save ~62% disk usage, very slow [not recommend]
  mintpy.load.processor      = aria  #[isce, aria, snap, gamma, roipac], auto for isce
  mintpy.load.updateMode     = auto  #[yes / no], auto for yes, skip re-loading if HDF5 files are complete
  mintpy.load.compression    = auto  #[gzip / lzf / no], auto for no.
  ##---------interferogram datasets:
  mintpy.load.unwFile        = ../stack/unwrapStack.vrt
  mintpy.load.corFile        = ../stack/cohStack.vrt
  mintpy.load.connCompFile   = ../stack/connCompStack.vrt
  mintpy.load.magFile        = ../stack/ampStack.vrt        # optional
  ##---------geometry datasets:
  mintpy.load.demFile        = ../DEM/SRTM_3arcsec.dem
  mintpy.load.incAngleFile   = ../incidenceAngle/*.vrt
  mintpy.load.azAngleFile    = ../azimuthAngle/*.vrt
  mintpy.load.waterMaskFile  = ../mask/watermask.msk
  ##---------subset (optional):
  ## if both yx and lalo are specified, use lalo option
  mintpy.subset.yx           = auto    #[y0:y1,x0:x1 / no], auto for no
  mintpy.subset.lalo         = auto    #[lat0:lat1,lon0:lon1 / no], auto for no
  ##---------multilook (optional):
  ## multilook while loading data with the specified method, to reduce dataset size
  ## nearest, mean and median methods are applicable to interferogram/ionosphere/offset stack(s), except for:
  ## connected components and all geometry datasets, for which nearest is hardwired.
  mintpy.multilook.method    = auto    #[nearest, mean, median], auto for nearest - lines/rows skipping approach
  mintpy.multilook.ystep     = auto    #[int >= 1], auto for 1 - no multilooking
  mintpy.multilook.xstep     = auto    #[int >= 1], auto for 1 - no multilooking
"""

EXAMPLE = """example:
  prep_aria.py -t smallbaselineApp.cfg    # recommended
  prep_aria.py -t SanFranSenDT42.txt
  prep_aria.py -s ../stack/ -d ../DEM/SRTM_3arcsec.dem -i '../incidenceAngle/*.vrt'
  prep_aria.py -s ../stack/ -d ../DEM/SRTM_3arcsec.dem -i '../incidenceAngle/*.vrt' -a '../azimuthAngle/*.vrt' -w ../mask/watermask.msk

  # download / extract / prepare inteferograms stack from ARIA using ARIA-tools:
  # reference: https://github.com/aria-tools/ARIA-tools
  ariaDownload.py -b '37.25 38.1 -122.6 -121.75' --track 42
  ariaTSsetup.py -f 'products/*.nc' -b '37.25 38.1 -122.6 -121.75' --mask Download --num_threads 4 --verbose
"""

def create_parser(subparsers=None):
    """Command line parser."""
    synopsis = 'Prepare ARIA processed products for MintPy.'
    epilog = TEMPLATE + '\n' + EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('-t','--template', dest='template_file', type=str,
                        help='template file with the options')
    parser.add_argument('-o', '--output', type=str, nargs=2, dest='outfile',
                        default=['./inputs/ifgramStack.h5',
                                 './inputs/geometryGeo.h5'],
                        help='output HDF5 file')
    parser.add_argument('--update', dest='updateMode', action='store_true',
                        help='Enable the update mode: checking dataset already loaded.')
    parser.add_argument('--compression', choices={'gzip', 'lzf', None}, default=None,
                        help='HDF5 file compression, default: %(default)s')

    # ifgramStack
    stack = parser.add_argument_group('interferogram stack')
    stack.add_argument('-s','--stack-dir', dest='stackDir', type=str,
                       help='The directory which contains stack VRT files.')
    stack.add_argument('-u','--unwrap-stack-name', dest='unwFile', type=str,
                       default="unwrapStack.vrt",
                       help='Name of the stack VRT file of unwrapped data.\n'+
                            'default: %(default)s')
    stack.add_argument('-c','--coherence-stack-name', dest='corFile', type=str,
                       default="cohStack.vrt",
                       help='Name of the stack VRT file of coherence data.\n'+
                            'default: %(default)s')
    stack.add_argument('-l','--conn-comp-stack-name', dest='connCompFile', type=str,
                       default="connCompStack.vrt",
                       help='Name of the stack VRT file of connected component data.\n' +
                            'default: %(default)s')
    stack.add_argument('--amp-stack-name','--amplitude-stack-name', dest='magFile', type=str,
                       default="ampStack.vrt",
                       help='Name of the stack VRT file of interferogram amplitude data (optional).\n' +
                            'default: %(default)s')

    # geometryGeo
    geom = parser.add_argument_group('geometry')
    geom.add_argument('-d','--dem', dest='demFile', type=str,
                      help='Name of the DEM file')
    geom.add_argument('-i','--incidence-angle', dest='incAngleFile', type=str,
                      help='Name of the incidence angle file')
    geom.add_argument('-a','--az-angle','--azimuth-angle', dest='azAngleFile', type=str,
                      help='Name of the azimuth angle file.')
    geom.add_argument('-w','--water-mask', dest='waterMaskFile', type=str,
                      help='Name of the water mask file')
    return parser


def cmd_line_parse(iargs = None):
    # parse
    parser = create_parser()
    inps = parser.parse_args(args=iargs)

    # default: multilook options
    iDict = vars(inps)
    iDict['xstep'] = int(iDict.get('xstep', 1))
    iDict['ystep'] = int(iDict.get('ystep', 1))
    iDict['method'] = str(iDict.get('method', 'nearest'))

    # check: --template option
    if inps.template_file:
        inps = read_template2inps(inps.template_file, inps)
    print('multilook x/ystep: {}/{}'.format(iDict['xstep'], iDict['ystep']))
    print('multilook method : {}'.format(iDict['method']))

    # check: --stack-dir
    if inps.stackDir is not None:
        inps.stackDir     = os.path.abspath(inps.stackDir)
        inps.corFile      = os.path.join(inps.stackDir, os.path.basename(inps.corFile))
        inps.unwFile      = os.path.join(inps.stackDir, os.path.basename(inps.unwFile))
        inps.connCompFile = os.path.join(inps.stackDir, os.path.basename(inps.connCompFile))

    # check: all options end with "File"
    # translate wildcard path input with search result
    # if not exist, raise error for required datasets
    #               set to None for the other datasets
    print('search input data file info:')
    ds_keys = [key for key in list(iDict.keys()) if key.endswith('File')]
    required_ds_keys = ['unwFile', 'corFile', 'demFile', 'incAngleFile']
    max_digit = max(len(i) for i in ds_keys)

    for key in ds_keys:
        # search for wildcard pattern
        fnames = glob.glob(iDict[key]) if iDict[key] else []

        # user the first element if more than one exist
        if len(fnames) > 0:
            iDict[key] = fnames[0]
            print('{k:<{w}} : {f}'.format(k=key, w=max_digit, f=fnames[0]))

        elif key in required_ds_keys:
            # raise exception if any required DS is missing
            raise SystemExit(f'ERROR: no file found for {key} in input path: "{iDict[key]}"!')

        else:
            iDict[key] = None

    return inps


def read_template2inps(template_file, inps):
    """Read input template file into inps"""
    print(f'read options from template file: {os.path.basename(template_file)}')

    from mintpy.utils import readfile, utils1 as ut

    iDict = vars(inps)
    template = readfile.read_template(template_file)
    template = ut.check_template_auto_value(template)

    # ignore template options with default auto values
    # so that options from input arguments have higher priority
    # than template options with auto values
    for key in list(template.keys()):
        if template[key] == 'auto':
            template.pop(key)

    # pass options from template to inps
    # group - load
    key_prefix = 'mintpy.load.'
    keys = [i for i in list(iDict.keys()) if key_prefix+i in template.keys()]
    for key in keys:
        value = template[key_prefix+key]
        if key in ['updateMode', 'compression']:
            iDict[key] = value
        elif key in ['unwFile']:
            iDict['stackDir'] = os.path.dirname(value)
        elif value:
            iDict[key] = str(value)

    # group - multilook
    prefix = 'mintpy.multilook.'
    key_list = [i.split(prefix)[1] for i in template.keys() if i.startswith(prefix)]
    for key in key_list:
        value = template[prefix+key]
        if key in ['xstep', 'ystep']:
            iDict[key] = int(template[prefix+key])
        elif key in ['method']:
            iDict[key] = template[prefix+key]

    return inps


####################################################################################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.prep_aria import load_aria

    # run
    load_aria(inps)


####################################################################################
if __name__=="__main__":
    main(sys.argv[1:])



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Zhang Yunjun, Aug 2022        #
############################################################


import os
import sys

from mintpy.defaults.template import get_template_content
from mintpy.utils.arg_utils import create_argument_parser

####################################################################################################
TEMPLATE = get_template_content('correct_unwrap_error')

REFERENCE = """reference:
  Yunjun, Z., H. Fattahi, and F. Amelung (2019), Small baseline InSAR time series analysis:
    Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
    doi:10.1016/j.cageo.2019.104331.
"""

EXAMPLE = """Example:
  unwrap_error_bridging.py  ./inputs/ifgramStack.h5  -t GalapagosSenDT128.template --update
  unwrap_error_bridging.py  ./inputs/ifgramStack.h5  --water-mask waterMask.h5
  unwrap_error_bridging.py  20180502_20180619.unw    --water-mask waterMask.h5
"""

NOTE = """
  by connecting reliable regions with MST bridges. This method assumes the phase differences
  between neighboring regions are less than pi rad in magnitude.
"""

def create_parser(subparsers=None):
    synopsis = 'Unwrapping Error Correction with Bridging'
    epilog = REFERENCE + '\n' + TEMPLATE + '\n' + EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis+NOTE, epilog=epilog, subparsers=subparsers)

    parser.add_argument('ifgram_file', type=str, help='interferograms file to be corrected')
    parser.add_argument('-r','--radius', dest='bridgePtsRadius', type=int, default=50,
                        help='radius of the end point of bridge to search area to get median representative value\n'+
                             'default: 50.')
    parser.add_argument('--ramp', dest='ramp', choices=['linear', 'quadratic'],
                          help='type of phase ramp to be removed before correction.')
    parser.add_argument('--water-mask','--wm', dest='waterMaskFile', type=str, help='path of water mask file.')
    parser.add_argument('-m', '--min-area', dest='connCompMinArea', type=float, default=2.5e3,
                        help='minimum region/area size of a single connComponent.')

    parser.add_argument('-t', '--template', dest='template_file', type=str,
                          help='template file with bonding point info, e.g.\n' +
                               'mintpy.unwrapError.yx = 283,1177,305,1247;350,2100,390,2200')

    parser.add_argument('-i','--in-dataset', dest='datasetNameIn', default='unwrapPhase',
                        help='name of dataset to be corrected, default: unwrapPhase')
    parser.add_argument('-o','--out-dataset', dest='datasetNameOut',
                        help='name of dataset to be written after correction, default: {}_bridging')
    parser.add_argument('--update', dest='update_mode', action='store_true',
                        help='Enable update mode: if unwrapPhase_unwCor dataset exists, skip the correction.')
    return parser


def cmd_line_parse(iargs=None):
    # parse
    parser = create_parser()
    inps = parser.parse_args(args=iargs)

    # import
    from mintpy.utils import readfile

    # check: input file type
    ftype = readfile.read_attribute(inps.ifgram_file)['FILE_TYPE']
    if ftype not in ['ifgramStack', '.unw']:
        raise ValueError(f'input file is not ifgramStack: {ftype}')

    # check: -t / --template option (read template content)
    if inps.template_file:
        inps = read_template2inps(inps.template_file, inps)

    # check: --water-mask option (file existence)
    if inps.waterMaskFile and not os.path.isfile(inps.waterMaskFile):
        inps.waterMaskFile = None

    # default: -o / --out-dataset option
    if not inps.datasetNameOut:
        inps.datasetNameOut = f'{inps.datasetNameIn}_bridging'

    return inps


def read_template2inps(template_file, inps):
    """Read input template options into Namespace inps"""
    print('read options from template file: '+os.path.basename(template_file))

    from mintpy.unwrap_error_bridging import key_prefix
    from mintpy.utils import readfile, utils1 as ut

    inpsDict = vars(inps)
    template = readfile.read_template(inps.template_file)
    template = ut.check_template_auto_value(template)

    key_list = [i for i in list(inpsDict.keys()) if key_prefix+i in template.keys()]
    for key in key_list:
        value = template[key_prefix+key]
        if key in ['update']:
            inpsDict[key] = value

        elif value:
            if key in ['waterMaskFile', 'ramp']:
                inpsDict[key] = value
            elif key in ['bridgePtsRadius']:
                inpsDict[key] = int(value)
            elif key in ['connCompMinArea']:
                inpsDict[key] = float(value)

    return inps


####################################################################################################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.unwrap_error_bridging import (
        run_or_skip,
        run_unwrap_error_bridging,
    )

    # run or skip
    if inps.update_mode and run_or_skip(inps) == 'skip':
        return

    # run
    run_unwrap_error_bridging(
        ifgram_file=inps.ifgram_file,
        water_mask_file=inps.waterMaskFile,
        ramp_type=inps.ramp,
        radius=inps.bridgePtsRadius,
        cc_min_area=inps.connCompMinArea,
        dsNameIn=inps.datasetNameIn,
        dsNameOut=inps.datasetNameOut,
        inps=inps,
    )


####################################################################################################
if __name__ == '__main__':
    main(sys.argv[1:])



############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Zhang Yunjun, Mar 2018                           #
############################################################
# recommended usage:
#   from mintpy.defaults import auto_path


import glob
import os
import re

# Default path of data files from different InSAR processors to be loaded into MintPy
AUTO_PATH_ISCE_TOPS = '''##----------Default file path of ISCE/topsStack products
mintpy.load.processor       = isce
mintpy.load.metaFile        = ../reference/IW*.xml
mintpy.load.baselineDir     = ../baselines

mintpy.load.unwFile         = ../merged/interferograms/*/filt*.unw
mintpy.load.corFile         = ../merged/interferograms/*/filt*.cor
mintpy.load.connCompFile    = ../merged/interferograms/*/filt*.unw.conncomp
mintpy.load.intFile         = None

mintpy.load.ionUnwFile      = ../ion/*/ion_cal/filt.ion
mintpy.load.ionCorFile      = ../ion/*/ion_cal/raw_no_projection.cor
mintpy.load.ionConnCompFile = None

mintpy.load.demFile         = ../merged/geom_reference/hgt.rdr
mintpy.load.lookupYFile     = ../merged/geom_reference/lat.rdr
mintpy.load.lookupXFile     = ../merged/geom_reference/lon.rdr
mintpy.load.incAngleFile    = ../merged/geom_reference/los.rdr
mintpy.load.azAngleFile     = ../merged/geom_reference/los.rdr
mintpy.load.shadowMaskFile  = ../merged/geom_reference/shadowMask.rdr
mintpy.load.waterMaskFile   = ../merged/geom_reference/waterMask.rdr
mintpy.load.bperpFile       = None
'''

AUTO_PATH_ISCE_STRIPMAP = '''##----------Default file path of ISCE/stripmapStack products
mintpy.load.processor       = isce
mintpy.load.metaFile        = ${m_shelve}/data.dat
mintpy.load.baselineDir     = ../baselines

mintpy.load.unwFile         = ../Igrams/*/filt*.unw
mintpy.load.corFile         = ../Igrams/*/filt*.cor
mintpy.load.connCompFile    = ../Igrams/*/filt*.unw.conncomp
mintpy.load.intFile         = None

mintpy.load.demFile         = ../geom_reference/hgt.rdr
mintpy.load.lookupYFile     = ../geom_reference/lat.rdr
mintpy.load.lookupXFile     = ../geom_reference/lon.rdr
mintpy.load.incAngleFile    = ../geom_reference/los.rdr
mintpy.load.azAngleFile     = ../geom_reference/los.rdr
mintpy.load.shadowMaskFile  = ../geom_reference/shadowMask.rdr
mintpy.load.waterMaskFile   = ../geom_reference/waterMask.rdr
mintpy.load.bperpFile       = None
'''

AUTO_PATH_ROIPAC = '''##----------Default file path of ROI_PAC products
mintpy.load.processor       = roipac
mintpy.load.unwFile         = ../PROCESS/DONE/IFG*/filt*.unw
mintpy.load.corFile         = ../PROCESS/DONE/IFG*/filt*.cor
mintpy.load.connCompFile    = ../PROCESS/DONE/IFG*/filt*snap_connect.byt

mintpy.load.demFile         = ../PROCESS/DONE/*${m_date12}*/radar_*rlks.hgt
mintpy.load.lookupYFile     = ../PROCESS/GEO/geo_${m_date12}/geomap_*rlks.trans
mintpy.load.lookupXFile     = ../PROCESS/GEO/geo_${m_date12}/geomap_*rlks.trans
mintpy.load.incAngleFile    = None
mintpy.load.azAngleFile     = None
mintpy.load.shadowMaskFile  = None
mintpy.load.bperpFile       = None
'''

AUTO_PATH_GAMMA = '''##----------Default file path of GAMMA products
mintpy.load.processor       = gamma
mintpy.load.unwFile         = ../PROCESS/DONE/IFG*/diff*rlks.unw
mintpy.load.corFile         = ../PROCESS/DONE/IFG*/*filt*rlks.cor
mintpy.load.connCompFile    = None

mintpy.load.demFile         = ../PROCESS/SIM/sim_${m_date12}/sim*.hgt_sim
mintpy.load.lookupYFile     = ../PROCESS/SIM/sim_${m_date12}/sim*.UTM_TO_RDC
mintpy.load.lookupXFile     = ../PROCESS/SIM/sim_${m_date12}/sim*.UTM_TO_RDC
mintpy.load.incAngleFile    = None
mintpy.load.azAngleFile     = None
mintpy.load.shadowMaskFile  = None
mintpy.load.bperpFile       = ../merged/baselines/*/*.base_perp
'''

AUTO_PATH_ARIA = '''##----------Default file path of ARIA products
mintpy.load.processor       = aria
mintpy.load.unwFile         = ../stack/unwrapStack.vrt
mintpy.load.corFile         = ../stack/cohStack.vrt
mintpy.load.connCompFile    = ../stack/connCompStack.vrt

mintpy.load.demFile         = ../DEM/*.dem
mintpy.load.lookupYFile     = None
mintpy.load.lookupXFile     = None
mintpy.load.incAngleFile    = ../incidenceAngle/*.vrt
mintpy.load.azAngleFile     = ../azimuthAngle/*.vrt
mintpy.load.shadowMaskFile  = None
mintpy.load.waterMaskFile   = ../mask/watermask.msk
'''


AUTO_PATH_DICT = {
    'isce_tops'     : AUTO_PATH_ISCE_TOPS,
    'isce_stripmap' : AUTO_PATH_ISCE_STRIPMAP,
    'roipac'        : AUTO_PATH_ROIPAC,
    'gamma'         : AUTO_PATH_GAMMA,
    'aria'          : AUTO_PATH_ARIA,
}

prefix = 'mintpy.load.'


##----------------- Functions from mintpy.utils.readfile to be independnt module ---------##
def read_str2dict(inString, delimiter='=', print_msg=False):
    '''Read multiple lines of string into dict
    Based on mintpy.utils.readfile.read_template()
    '''
    strDict = {}
    lines = inString.split('\n')
    for line in lines:
        c = [i.strip() for i in line.strip().split(delimiter, 1)]
        if len(c) < 2 or line.startswith(('%', '#')):
            next
        else:
            key = c[0]
            value = str.replace(c[1], '\n', '').split("#")[0].strip()
            if value != '':
                strDict[key] = value

    # set 'None' to None
    for key, value in strDict.items():
        if value.lower() == 'none':
            strDict[key] = None
    return strDict


##----------------------------------------------------------------------------------------##
def get_auto_path(processor, work_dir, template):
    """Update template options with auto path defined in AUTO_PATH_DICT
    Parameters: processor - str, isce / roipac / gamma
                work_dir  - str, mintpy work directory, e.g. ./GalapagosSenDT128/mintpy
                template  - dict,
    Returns:    template  - dict,
    """
    ## 1. *AutoPath --> auto_path_dict
    proj_dir = os.path.dirname(work_dir)

    # specific stack processor within ISCE
    if processor == 'isce':
        if os.path.exists(proj_dir + '/reference'):
            processor = 'isce_tops'
        elif os.path.exists(proj_dir + '/Igrams'):
            processor = 'isce_stripmap'
        else:
            raise ValueError('un-recognized ISCE file directory, thus, cannot use auto path setting!')

    # read auto_path_dict
    auto_path_dict = read_str2dict(AUTO_PATH_DICT[processor], print_msg=False)

    ## 2. translate variables in *AutoPath
    ## e.g.: m_shelve, m_date12
    var_dict = {}

    if processor in ['roipac', 'gamma']:
        m_date12 = get_reference_date12(proj_dir, processor)
        if m_date12:
            var_dict['${m_date12}'] = m_date12

        dem_file = get_dem_file(proj_dir, m_date12, processor)
        if dem_file:
            auto_path_dict[prefix+'demFile'] = dem_file

    elif processor == 'isce_stripmap':
        date_str = os.listdir(os.path.join(proj_dir, 'merged/SLC'))[0]
        var_dict['${m_shelve}'] = os.path.join(proj_dir, 'merged/SLC', date_str, 'referenceShelve')

    # update auto_path_dict
    for key, value in auto_path_dict.items():
        if value:
            for var1, var2 in var_dict.items():
                value = value.replace(var1, var2)
            auto_path_dict[key] = value

    ## 3. update input template option with auto value
    max_digit = max(len(key) for key in auto_path_dict.keys())
    for key, value in auto_path_dict.items():
        if value and template[key] == 'auto':
            template[key] = value
            print('    {k:<{d}} : auto --> {v}'.format(d=max_digit, k=key, v=value))

    return template


def get_reference_date12(proj_dir, processor='roipac'):
    """date12 of reference interferogram in YYMMDD-YYMMDD format"""
    import numpy as np

    m_date12 = None

    # opt 1 - reference_ifgram.txt
    m_ifg_file = os.path.join(proj_dir, 'PROCESS', 'reference_ifgram.txt')
    if os.path.isfile(m_ifg_file):
        m_date12 = str(np.loadtxt(m_ifg_file, dtype=bytes).astype(str))
        return m_date12

    # opt 2 - folders under GEO/SIM
    if processor == 'roipac':
        try:
            lookup_file = glob.glob(os.path.join(proj_dir, 'PROCESS/GEO/geo_*/geomap*.trans'))[0]
            m_date12 = re.findall(r'\d{6}-\d{6}', lookup_file)[0]
        except:
            print("No reference interferogram found! Check the PROCESS/GEO/geo_* folder")

    elif processor == 'gamma':
        geom_dir = os.path.join(proj_dir, 'PROCESS/SIM')
        try:
            m_date12 = os.walk(geom_dir).next()[1][0].split('sim_')[1]
        except:
            print("No reference interferogram found! Check the PROCESS/SIM/sim_* folder")

    return m_date12


def get_dem_file(proj_dir, m_date12, processor):
    """get DEM file in case both radar_2rlks.hgt and radar_8rlks.hgt exist"""
    dem_file = None

    if m_date12 and processor == 'roipac':
        # get the number of looks used in lookup table file
        lookup_file = os.path.join(proj_dir, f'PROCESS/GEO/geo_{m_date12}/geomap*.trans')
        lks = re.findall(r'_\d+rlks', glob.glob(lookup_file)[0])[0]

        # use the one with same multilook info as the lookup table file.
        dem_file = os.path.join(proj_dir, 'PROCESS/DONE/*${m_date12}*', f'radar{lks}.hgt')

    return dem_file



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Zhang Yunjun, Aug 2022        #
############################################################


import os
import sys

from mintpy.utils.arg_utils import create_argument_parser

###########################  Sub Function  ##############################
BL_LIST = """
070106     0.0   0.03  0.0000000  0.00000000000 2155.2 /scratch/SLC/070106/
070709  2631.9   0.07  0.0000000  0.00000000000 2155.2 /scratch/SLC/070709/
070824  2787.3   0.07  0.0000000  0.00000000000 2155.2 /scratch/SLC/070824/
"""

DATE12_LIST = """
20070709_20100901
20070709_20101017
20070824_20071009
"""

TEMPLATE = """
mintpy.network.maskFile  = auto  #[file name, no], auto for waterMask.h5 or no for all pixels
mintpy.network.aoiYX     = auto  #[y0:y1,x0:x1 / no], auto for no, area of interest for coherence calculation
mintpy.network.aoiLALO   = auto  #[lat0:lat1,lon0:lon1 / no], auto for no - use the whole area
"""

EXAMPLE = """example:
  plot_network.py inputs/ifgramStack.h5
  plot_network.py inputs/ifgramStack.h5 -t smallbaselineApp.cfg --nodisplay   #Save figures to files without display
  plot_network.py inputs/ifgramStack.h5 -t smallbaselineApp.cfg --show-kept   #Do not plot dropped ifgrams
  plot_network.py inputs/ifgramStack.h5 -d tbase -v 0 365.25 -c RdYlBu_r      #Color-code lines by temporal      baseline
  plot_network.py inputs/ifgramStack.h5 -d pbase -v 0 180    -c RdYlBu_r      #Color-code lines by perpendicular baseline
  plot_network.py coherenceSpatialAvg.txt

  # offsetSNR
  plot_network.py inputs/ifgramStack.h5 -d offsetSNR -v 0 20 --cmap-vlist 0 0.2 1
"""


def create_parser(subparsers=None):
    synopsis = 'Display Network of Interferograms'
    epilog = EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('file', help='file with network information, ifgramStack.h5 or coherenceSpatialAvg.txt')
    parser.add_argument('--show-kept', dest='disp_drop', action='store_false',
                        help='display kept interferograms only, without dropped interferograms')

    # Color-code coherence/baseline in the network/matrix plot
    color = parser.add_argument_group('Color-code network/matrix plot',
                                      'color-code phase/offset pairs with coherence/baseline in network/matrix plot')
    color.add_argument('-d', '--dset', type=str, dest='dsetName', default='coherence',
                       choices={'coherence','offsetSNR','pbase','tbase'},
                       help='dataset used to calculate the mean. (default: %(default)s)')
    color.add_argument('-v', '--vlim', nargs=2, type=float, default=(0.2, 1.0), help='display range')

    color.add_argument('-t', '--template', dest='template_file',
                       help='template file with options below:\n'+TEMPLATE)
    color.add_argument('--mask', dest='maskFile', default='waterMask.h5',
                       help='mask file used to calculate the coherence. Default: waterMask.h5 or None.')

    color.add_argument('-c', '--colormap', dest='cmap_name', default='RdBu_truncate',
                       help='colormap name for the network display. Default: RdBu_truncate')
    color.add_argument('--cmap-vlist', dest='cmap_vlist', type=float, nargs=3, default=[0.2, 0.4, 1.0],
                       help='normalized start/jump/end value for truncated colormap (default: %(default)s).')

    # Figure  Setting
    fig = parser.add_argument_group('Figure', 'Figure settings for display')
    fig.add_argument('--fs', '--fontsize', type=int,
                     default=12, help='font size in points')
    fig.add_argument('--lw', '--linewidth', dest='linewidth',
                     type=int, default=2, help='line width in points')
    fig.add_argument('--mc', '--markercolor', dest='markercolor',
                     default='orange', help='marker color')
    fig.add_argument('--ms', '--markersize', dest='markersize',
                     type=int, default=8, help='marker size in points (default: %(default)s).')
    fig.add_argument('--every-year', dest='every_year', type=int,
                     default=1, help='number of years per major tick on x-axis')

    fig.add_argument('--dpi', dest='fig_dpi', type=int, default=1200,
                     help='DPI - dot per inch - for display/write')
    fig.add_argument('--figsize', dest='fig_size', type=float, nargs=2,
                     help='figure size in inches - width and length.')
    fig.add_argument('--notitle', dest='disp_title', action='store_false',
                     help='Do not display figure title.')
    fig.add_argument('--number', dest='number', type=str,
                     help='number mark to be plot at the corner of figure.')
    fig.add_argument('--nosplit-cmap', dest='split_cmap', action='store_false',
                     help='do not split colormap for coherence color')

    fig.add_argument('--save', dest='save_fig',
                     action='store_true', help='save the figure')
    fig.add_argument('--nodisplay', dest='disp_fig',
                     action='store_false', help='save and do not display the figure')
    return parser


def cmd_line_parse(iargs=None):
    # parse
    parser = create_parser()
    inps = parser.parse_args(args=iargs)

    # import
    from mintpy.utils import readfile

    # save argv (to check the manually specified arguments)
    # use iargs        for python call
    # use sys.argv[1:] for command line call
    inps.argv = iargs if iargs else sys.argv[1:]

    # check: input file type
    if inps.file.endswith(('.h5','.he5')):
        k = readfile.read_attribute(inps.file)['FILE_TYPE']
        if k != 'ifgramStack':
            raise ValueError('input HDF5 file is NOT ifgramStack.')

    # check: --nodisplay option
    if not inps.disp_fig:
        inps.save_fig = True

    # check: -t / --template option
    if inps.template_file:
        inps = read_template2inps(inps.template_file, inps)
    else:
        inps.template = {}

    # check: --mask option (file existence)
    if not os.path.isfile(inps.maskFile):
        inps.maskFile = None

    return inps


def read_template2inps(template_file, inps):
    """Read input template options into Namespace inps"""
    print('read options from template file: '+os.path.basename(template_file))

    from mintpy.utils import readfile, utils1 as ut

    inps.template = readfile.read_template(inps.template_file)
    inps.template = ut.check_template_auto_value(inps.template)

    # coherence-based network modification
    prefix = 'mintpy.network.'
    key = prefix+'maskFile'
    if key in inps.template.keys():
        if inps.template[key]:
            inps.maskFile = inps.template[key]

    return inps


##########################  Main Function  ##############################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.plot_network import plot_network

    # run
    plot_network(inps)


#########################################################################
if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
# Author: Yuan-Kai Liu, Zhang Yunjun, Oct 2022
"""Test mintpy.objects.euler module for the Euler pole and velocity computation."""


import collections
import math

from mintpy.cli.plate_motion import ITRF2014_PMM
from mintpy.objects.euler_pole import MASY2DMY, EulerPole

# validation against the UNAVCO Plate Motion Calculator
# https://www.unavco.org/software/geodetic-utilities/plate-motion-calculator/plate-motion-calculator.html
# accessed on Oct 29, 2022.
# config: height=0, model=ITRF2014, reference=NNR no-net-rotation
# note: azimuth here is measured from the north with positive for clock-wise direction
# unit:                              deg deg mm/yr   deg   mm/yr mm/yr
Tag = collections.namedtuple('Tag', 'lat lon speed azimuth vel_n vel_e')
POINT_PM = {
    'AUST' : Tag(-24,  132,  67.56,  28.94,  59.12,  32.69),
    'EURA' : Tag( 27,   62,  28.85,  79.21,   5.40,  28.34),
    'ARAB' : Tag( 18,   48,  46.51,  50.92,  29.32,  36.11),
}
PLATE_NAMES = POINT_PM.keys()


def test_euler_pole_initiation():
    print('Test 1: EulerPole object initiation and vector2pole conversion.')

    for plate_name in PLATE_NAMES:
        print(f'Plate name: ITRF2014-PMM {plate_name}')

        # get PMM info from Table 1 in Altamimi et al. (2017) as reference
        plate_pmm = ITRF2014_PMM[plate_name]

        # build EulerPole obj
        pole_obj = EulerPole(wx=plate_pmm.omega_x, wy=plate_pmm.omega_y, wz=plate_pmm.omega_z)

        # compare rotation rate: ITRF2014_PMM vs. Euler vector to pole conversion
        print(f'Reference  rotation rate from Altamimi et al. (2017): {plate_pmm.omega:.4f} deg/Ma')
        print(f'Calculated rotation rate from pole2vector conversion: {plate_pmm.omega:.4f} deg/Ma')
        assert math.isclose(plate_pmm.omega, pole_obj.rotRate*MASY2DMY, abs_tol=5e-4)
        print('Pass.')


def test_plate_motion_calc():
    print('Test 2: Plate motion calculation and validation against UNAVCO website.')

    for plate_name in PLATE_NAMES:
        # get UNAVCO result as reference
        point_pm = POINT_PM[plate_name]
        print(f'Plate = ITRF2014-PMM {plate_name}, point lat/lon = {point_pm.lat}/{point_pm.lon}')

        # calculate using EulerPole in m/yr
        plate_pmm = ITRF2014_PMM[plate_name]
        pole_obj = EulerPole(wx=plate_pmm.omega_x, wy=plate_pmm.omega_y, wz=plate_pmm.omega_z)
        ve, vn = pole_obj.get_velocity_enu(point_pm.lat, point_pm.lon, print_msg=False)[:2]

        print(f'Reference   (UNAVCO): vel_e={point_pm.vel_e:.2f}, vel_n={point_pm.vel_n:.2f} mm/yr')
        print(f'Calculation (MintPy): vel_e={ve*1e3:.2f}, vel_n={vn*1e3:.2f} mm/yr')
        assert math.isclose(point_pm.vel_e, ve*1e3, abs_tol=0.05)
        assert math.isclose(point_pm.vel_n, vn*1e3, abs_tol=0.05)
        print('Pass.')


if __name__ == '__main__':

    print('-'*50)
    print(f'Testing {__file__}')

    test_euler_pole_initiation()

    test_plate_motion_calc()



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Zhang Yunjun, Aug 2022        #
############################################################


import os
import sys

from mintpy.defaults.template import get_template_content
from mintpy.utils.arg_utils import create_argument_parser

###############################  Usage  ################################
TEMPLATE = get_template_content('modify_network')

REFERENCE = """reference:
  Yunjun, Z., Fattahi, H. and Amelung, F. (2019), Small baseline InSAR time series analysis:
    Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
    doi:10.1016/j.cageo.2019.104331.
  Chaussard, E., Bürgmann, R., Fattahi, H., Nadeau, R. M., Taira, T., Johnson, C. W. and Johanson, I.
    (2015), Potential for larger earthquakes in the East San Francisco Bay Area due to the direct
    connection between the Hayward and Calaveras Faults, Geophysical Research Letters, 42(8),
    2734-2741, doi:10.1002/2015GL063575.
  Kang, Y., Lu, Z., Zhao, C., Xu, Y., Kim, J. W., & Gallegos, A. J. (2021).InSAR monitoring
    of creeping landslides in mountainous regions: A case study in Eldorado National Forest,
    California. Remote Sensing of Environment, 258, 112400. doi:10.1016/j.rse.2021.112400
"""

EXAMPLE = """example:
  modify_network.py inputs/ifgramStack.h5 -t smallbaselineApp.cfg
  modify_network.py inputs/ifgramStack.h5 --reset
  modify_network.py inputs/ifgramStack.h5 --manual
"""


def create_parser(subparsers=None):
    synopsis = 'Modify the network of interferograms'
    epilog = REFERENCE + '\n' + TEMPLATE + '\n' + EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('file', help='Files to modify/drop network, e.g. inputs/ifgramStack.h5.')
    parser.add_argument('-t', '--template', dest='template_file',
                        help='Template file with input options')
    parser.add_argument('--reset', action='store_true',
                        help='restore all interferograms in the file, by marking all dropIfgram=True')
    parser.add_argument('--noaux', dest='update_aux', action='store_false',
                        help='Do not update auxiliary files, e.g.\n' +
                             'maskConnComp.h5 or avgSpatialCoh.h5 from ifgramStack.h5')

    # 1. temp/perp baseline, num of conn., dates, pair index, etc.
    parser.add_argument('--max-tbase', dest='tempBaseMax',
                        type=float, help='max temporal baseline in days')
    parser.add_argument('--max-pbase', dest='perpBaseMax',
                        type=float, help='max perpendicular baseline in meters')
    parser.add_argument('--max-conn-num', dest='connNumMax', type=int,
                        help='max number of connections/neighbors per acquisition')
    parser.add_argument('-r', '--reference', dest='referenceFile',
                        help='Reference hdf5 / list file with network information.\n'
                             'i.e. ifgramStack.h5, date12_list.txt')
    parser.add_argument('--exclude-ifg-index', dest='excludeIfgIndex', nargs='*',
                        help='index of interferograms to remove/drop.\n1 as the first')
    parser.add_argument('--exclude-date', dest='excludeDate', nargs='*',
                        help='date(s) to remove/drop, all interferograms included date(s) will be removed')
    parser.add_argument('--start-date', '--min-date', dest='startDate',
                        help='remove/drop interferograms with date earlier than start-date in YYMMDD or YYYYMMDD format')
    parser.add_argument('--end-date', '--max-date', dest='endDate',
                        help='remove/drop interferograms with date later than end-date in YYMMDD or YYYYMMDD format')

    # 2. coherence-based network
    cohBased = parser.add_argument_group('Data-driven network modification', 'Drop/modify network based on data')
    # 2.1 coherence-based
    cohBased.add_argument('--coherence-based', dest='coherenceBased', action='store_true',
                          help='Enable coherence-based network modification (default: %(default)s).')
    cohBased.add_argument('--min-coherence', dest='minCoherence', type=float, default=0.7,
                          help='Minimum coherence value (default: %(default)s).')
    # 2.2 area-ratio-based
    cohBased.add_argument('--area-ratio-based', dest='areaRatioBased', action='store_true',
                          help='Enable area ratio-based network modification (default: %(default)s).')
    cohBased.add_argument('--min-area-ratio', dest='minAreaRatio', type=float, default=0.75,
                          help='Minimum area ratio value (default: %(default)s).')
    # common parameters
    cohBased.add_argument('--no-mst', dest='keepMinSpanTree', action='store_false',
                          help='Do not keep interferograms in Min Span Tree network based on inversed mean coherene')
    cohBased.add_argument('--mask', dest='maskFile', default='waterMask.h5',
                          help='Mask file used to calculate the spatial coherence '
                               '(default: waterMask.h5 or None)')
    cohBased.add_argument('--aoi-yx', dest='aoi_pix_box', type=int, nargs=4, metavar=('X0', 'Y0', 'X1', 'Y1'), default=None,
                          help='AOI in row/column range for coherence calculation (default: %(default)s).')
    cohBased.add_argument('--aoi-lalo', dest='aoi_geo_box', type=float, nargs=4, metavar=('W', 'S', 'E', 'N'), default=None,
                          help='AOI in lat/lon range for coherence calculation (default: %(default)s).')
    cohBased.add_argument('--lookup', dest='lookupFile',
                          help='Lookup table/mapping transformation file for geo/radar coordinate conversion.\n' +
                               'Needed for mask AOI in lalo')

    # 3. manual selection
    manual = parser.add_argument_group('Manual Network', 'Manually select/drop/modify network')
    manual.add_argument('--manual', action='store_true',
                        help='display network to manually choose line/interferogram to remove')
    return parser


def cmd_line_parse(iargs=None):
    # parse
    parser = create_parser()
    inps = parser.parse_args(args=iargs)

    # import
    from mintpy.utils import utils as ut

    # check: --mask option
    if not os.path.isfile(inps.maskFile):
        inps.maskFile = None

    # check: --exclude-ifg-index option (convert input index to continous index list)
    inps.excludeIfgIndex = read_input_index_list(inps.excludeIfgIndex, stackFile=inps.file)

    # check: -t / --template option
    if inps.template_file:
        inps = read_template2inps(inps.template_file, inps)

    # check: input arguments (required at least one)
    required_args = [
        inps.referenceFile, inps.tempBaseMax, inps.perpBaseMax, inps.connNumMax,
        inps.excludeIfgIndex, inps.excludeDate, inps.coherenceBased, inps.areaRatioBased,
        inps.startDate, inps.endDate, inps.reset, inps.manual,
    ]
    if all(not i for i in required_args + [inps.template_file]):
        msg = 'No input option found to remove interferogram, exit.\n'
        msg += 'To manually modify network, please use --manual option '
        raise Exception(msg)

    # default: --lookup option
    if not inps.lookupFile:
        inps.lookupFile = ut.get_lookup_file()

    # default: turn --reset ON if:
    # 1) no input options found to drop ifgram AND
    # 2) there is template input
    if inps.template_file and all(not i for i in required_args):
        print('No input option found to remove interferogram')
        print('Keep all interferograms by enable --reset option')
        inps.reset = True

    return inps


def read_template2inps(template_file, inps):
    """Read input template options into Namespace inps"""
    print('read options from template file: '+os.path.basename(template_file))

    from mintpy.utils import ptime, readfile, utils as ut

    iDict = vars(inps)
    template = readfile.read_template(inps.template_file, skip_chars=['[', ']'])
    template = ut.check_template_auto_value(template)

    # Update inps if key existed in template file
    prefix = 'mintpy.network.'
    key_list = [i for i in list(iDict.keys()) if prefix+i in template.keys()]
    for key in key_list:
        value = template[prefix+key]
        if key in ['coherenceBased', 'areaRatioBased', 'keepMinSpanTree']:
            iDict[key] = value

        elif value:
            if key in ['minCoherence', 'minAreaRatio', 'tempBaseMax', 'perpBaseMax']:
                iDict[key] = float(value)
            elif key in ['connNumMax']:
                iDict[key] = int(value)
            elif key in ['maskFile', 'referenceFile']:
                iDict[key] = value

            elif key == 'aoiYX':
                tmp = [i.strip() for i in value.split(',')]
                sub_y = sorted(int(i.strip()) for i in tmp[0].split(':'))
                sub_x = sorted(int(i.strip()) for i in tmp[1].split(':'))
                inps.aoi_pix_box = (sub_x[0], sub_y[0], sub_x[1], sub_y[1])

            elif key == 'aoiLALO':
                tmp = [i.strip() for i in value.split(',')]
                sub_lat = sorted(float(i.strip()) for i in tmp[0].split(':'))
                sub_lon = sorted(float(i.strip()) for i in tmp[1].split(':'))
                inps.aoi_geo_box = (sub_lon[0], sub_lat[1], sub_lon[1], sub_lat[0])

                # Check lookup file
                if not inps.lookupFile:
                    print(f'Warning: NO lookup table file found! Can not use {key} option without it.')
                    print('Skip this option.')
                    inps.aoi_pix_box = None

            elif key in ['startDate', 'endDate']:
                iDict[key] = ptime.yyyymmdd(value)
            elif key == 'excludeDate':
                iDict[key] = ptime.yyyymmdd(value.split(','))
            elif key == 'excludeIfgIndex':
                iDict[key] += value.split(',')
                iDict[key] = read_input_index_list(iDict[key], stackFile=inps.file)

    return inps


def read_input_index_list(idxList, stackFile=None):
    """Read ['2','3:5','10'] into ['2','3','4','5','10']"""
    from mintpy.objects import ifgramStack

    if not idxList:
        return []

    idxListOut = []
    for idx in idxList:
        c = sorted(int(i) for i in idx.split(':'))
        if len(c) == 2:
            idxListOut += list(range(c[0], c[1]+1))
        elif len(c) == 1:
            idxListOut.append(c[0])
        else:
            print('Unrecoganized input: '+idx)
    idxListOut = sorted(set(idxListOut))

    # remove index not existed in the input ifgram stack file
    if stackFile:
        obj = ifgramStack(stackFile)
        obj.open(print_msg=False)
        idxListOut = [i for i in idxListOut if i < obj.numIfgram]

    return idxListOut


#########################  Main Function  ##############################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.modify_network import modify_network

    # run
    modify_network(inps)


########################################################################
if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Zhang Yunjun, Antonio Valentino, Aug 2016        #
############################################################


import sys

from mintpy.utils.arg_utils import create_argument_parser

#################################  Usage  ####################################
EXAMPLE = """example:
  spatial_average.py inputs/ifgramStack.h5  -d coherence -m maskConnComp.h5
  spatial_average.py timeseries_ERA5_demErr.h5 -m maskTempCoh.h5
"""

def create_parser(subparsers=None):
    synopsis = 'Calculate the spatial average.'
    epilog = EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('file', help='File to calculate spatial average')
    parser.add_argument('-d', '--dset', '--dataset', dest='datasetName',
                        help='dataset used to calculate, for ifgramStack file only.')
    parser.add_argument('-m', '--mask', dest='mask_file',
                        help='Mask file for the calculation')
    parser.add_argument('--nodisplay', dest='disp_fig',
                        action='store_false', help='save and do not display the figure')
    return parser


def cmd_line_parse(iargs=None):
    parser = create_parser()
    inps = parser.parse_args(args=iargs)
    return inps


##############################  Sub Function  ################################
def plot_spatial_average_ts(date_list:list, avg_list:list):
    """Plot the spatial average time-series."""
    import matplotlib.pyplot as plt

    from mintpy.utils import plot as pp, ptime

    dates = ptime.date_list2vector(date_list)[0]

    # plot
    _, ax = plt.subplots()
    ax.plot(dates, avg_list, '-o')

    # axis format
    ax = pp.auto_adjust_xaxis_date(ax, dates)[0]
    ax.set_title('Spatial Average', fontsize=12)
    ax.set_xlabel('Time [years]', fontsize=12)
    ax.set_ylabel('Mean', fontsize=12)
    plt.show()

    return


#############################  Main Function  ################################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.utils import readfile, utils1 as ut

    # run
    avg_list, date_list = ut.spatial_average(
        inps.file,
        datasetName=inps.datasetName,
        maskFile=inps.mask_file,
        saveList=True,
    )

    ftype = readfile.read_attribute(inps.file)['FILE_TYPE']
    if inps.disp_fig and ftype == 'timeseries':
        plot_spatial_average_ts(date_list, avg_list)

    return


##############################################################################
if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Zhang Yunjun, Aug 2022        #
############################################################


import sys

from mintpy.defaults.template import get_template_content
from mintpy.utils.arg_utils import create_argument_parser

######################################################################################################
TEMPLATE = get_template_content('residual_RMS')

REFERENCE="""reference:
  Yunjun, Z., Fattahi, H. and Amelung, F. (2019), Small baseline InSAR time series analysis:
    Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
    doi:10.1016/j.cageo.2019.104331.
  Rousseeuw, P. J., and M. Hubert (2011), Robust statistics for outlier detection,
    Wiley Interdisciplinary Reviews: Data Mining and Knowledge Discovery, 1(1),
    73-79, doi:doi:10.1002/widm.2.
"""

EXAMPLE = """example:
  timeseries_rms.py  timeseriesResidual.h5
  timeseries_rms.py  timeseriesResidual.h5  --template smallbaselineApp.cfg
  timeseries_rms.py  timeseriesResidual.h5  -m maskTempCoh.h5  --cutoff 3
"""


def create_parser(subparsers=None):
    synopsis = 'Calculate Root Mean Square (RMS) of deramped residual phase time-series.'
    epilog = TEMPLATE + '\n' + EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('timeseries_file', help='Timeseries file')
    parser.add_argument('-t', '--template', dest='template_file',
                        help='template file with options')
    parser.add_argument('-m', '--mask', dest='maskFile', default='maskTempCoh.h5',
                        help='mask file for estimation')
    parser.add_argument('-r','--ramp','--deramp', dest='deramp', default='quadratic',
                        help='ramp type to be remove for RMS calculation.\n' +
                             'Default - quadratic; no - do not remove ramp')
    parser.add_argument('--cutoff', dest='cutoff', default='3', type=float,
                        help='M-score used for outlier detection based on standardised residuals\n'+
                             'Recommend range: [3, 4], default is 3.')
    parser.add_argument('--figsize', dest='fig_size', metavar=('WID', 'LEN'),
                        type=float, nargs=2, default=[5., 3.],
                        help='figure size in inches - width and length')
    parser.add_argument('--tick-year-num', dest='tick_year_num',
                        type=int, default=1, help='Year number per major tick')
    return parser


def cmd_line_parse(iargs=None):
    """Command line parser."""
    parser = create_parser()
    inps = parser.parse_args(args=iargs)
    return inps


######################################################################################################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.timeseries_rms import run_timeseries_rms

    # run
    run_timeseries_rms(inps)


######################################################################################################
if __name__ == '__main__':
    main(sys.argv[1:])



############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Yunmeng Cao, Jul 2019                            #
############################################################


import os
import sys

import h5py
import numpy as np
from scipy.interpolate import griddata

try:
    from tqdm import tqdm
except ImportError:
    raise ImportError('Can not import tqdm!')

try:
    from concurrent.futures import ProcessPoolExecutor, as_completed
except ImportError:
    raise ImportError('Can not import concurrent!')

from mintpy.utils import readfile


################################################################################
def write_h5(datasetDict, out_file, metadata=None, ref_file=None, compression=None):

    if os.path.isfile(out_file):
        print(f'delete exsited file: {out_file}')
        os.remove(out_file)

    with h5py.File(out_file, 'w') as f:
        for dsName in datasetDict.keys():
            data = datasetDict[dsName]
            f.create_dataset(dsName,data=data,compression=compression)

        for key, value in metadata.items():
            f.attrs[key] = str(value)
            #print(key + ': ' +  value)
    print(f'finished writing to {out_file}')

    return out_file


def get_dataNames(FILE):
    with h5py.File(FILE, 'r') as f:
        dataNames = []
        for k0 in f.keys():
            dataNames.append(k0)
    return dataNames


def parallel_process(array, function, n_jobs=16, use_kwargs=False, front_num=1):
    """A parallel version of the map function with a progress bar.

        Args:
            array (array-like): An array to iterate over.
            function (function): A python function to apply to the elements of array
            n_jobs (int, default=16): The number of cores to use
            use_kwargs (boolean, default=False): Whether to consider the elements of array as dictionaries of
                keyword arguments to function
            front_num (int, default=3): The number of iterations to run serially before kicking off the parallel job.
                Useful for catching bugs
        Returns:
            [function(array[0]), function(array[1]), ...]
    """
    #We run the first few iterations serially to catch bugs
    if front_num > 0:
        front = [function(**a) if use_kwargs else function(a) for a in array[:front_num]]
    #If we set n_jobs to 1, just run a list comprehension. This is useful for benchmarking and debugging.
    if n_jobs==1:
        return front + [function(**a) if use_kwargs else function(a) for a in tqdm(array[front_num:])]
    #Assemble the workers
    with ProcessPoolExecutor(max_workers=n_jobs) as pool:
        #Pass the elements of array into function
        if use_kwargs:
            futures = [pool.submit(function, **a) for a in array[front_num:]]
        else:
            futures = [pool.submit(function, a) for a in array[front_num:]]
        kwargs = {
            'total': len(futures),
            'unit': 'it',
            'unit_scale': True,
            'leave': True
        }
        #Print out the progress as tasks complete
        for f in tqdm(as_completed(futures), **kwargs):
            del f
            #pass
    out = []
    #Get the results from the futures.
    for i, future in tqdm(enumerate(futures)):
        del i
        try:
            out.append(future.result())
        except Exception as e:
            out.append(e)
    return front + out


def split_range(N, M):
    #list0 = np.arange(0,N)
    dx = round(N/M)
    list00 = []
    for i in range(M):
        a0 = i*dx
        b0 = (i+1)*dx

        if b0 > N:
            b0 = N

        l0 = np.arange(a0,b0)
        list00.append(l0)

    return list00


def split_box(data,row_sample,col_sample):
    data_split= []
    row,col = data.shape
    list_row = split_range(row, row_sample)
    list_col = split_range(col, col_sample)

    for i in range(row_sample):
        for j in range(col_sample):
            y0 = min(list_row[i])
            y1 = max(list_row[i])
            x0 = min(list_col[j])
            x1 = max(list_col[j])

            data0 = data[y0:y1+1,x0:x1+1]
            data_split.append(data0)

    return data_split


def function(data0):
    points, zz1, zz2, grid_x0, grid_y0 = data0
    grid_lat0 = griddata(points, zz1, (grid_x0, grid_y0), method='nearest')
    grid_lon0 = griddata(points, zz2, (grid_x0, grid_y0), method='nearest')
    return grid_lat0, grid_lon0


def run_lookup_geo2radar(inps):
    """Convert the lookup table in geo-coordinates (from roipac, gamma) into radar-coordinates (from isce)."""

    rangeCoord = readfile.read(inps.geom_geo_file, datasetName = 'rangeCoord')[0].astype(np.float64)
    azimuthCoord = readfile.read(inps.geom_geo_file, datasetName = 'azimuthCoord')[0].astype(np.float64)
    #CPX_lt = complex(rangeCoord + '+' + azimuthCoord+'j')
    #CPX_lt = rangeCoord  + 1j *azimuthCoord

    meta_geo = readfile.read_attribute(inps.geom_geo_file)
    post_Lat = meta_geo['Y_STEP']
    post_Lon = meta_geo['X_STEP']
    Corner_LAT = meta_geo['Y_FIRST']
    Corner_LON = meta_geo['X_FIRST']

    if inps.write:
        meta = readfile.read_attribute(inps.write)
    elif inps.reference:
        meta = readfile.read_attribute(inps.reference)
    else:
        print('write_file or the reference_file should be provided at least one.')
        sys.exit(1)

    WIDTH_geo = int(meta_geo['WIDTH'])
    LENGTH_geo = int(meta_geo['LENGTH'])

    x = np.arange(0,WIDTH_geo)
    y = np.arange(0,LENGTH_geo)
    xv, yv = np.meshgrid(x, y)

    LAT = float(Corner_LAT) + yv*float(post_Lat)
    LON = float(Corner_LON) + xv*float(post_Lon)
    LAT = LAT.flatten()
    LON = LON.flatten()

    WIDTH  = int(meta['WIDTH'])
    LENGTH  = int(meta['LENGTH'])

    xx0 = rangeCoord.flatten()
    yy0 = azimuthCoord.flatten()

    zz01 = LAT.flatten()
    zz02 = LON.flatten()

    xx = xx0[xx0!=0]
    yy = yy0[xx0!=0]
    zz1 = zz01[xx0!=0] #lat
    zz2 = zz02[xx0!=0] # lon

    #points = (xx,yy)
    #points = np.zeros((len(xx),2))
    #points[:,0] = xx
    #points[:,1] = yy

    x = np.arange(0,WIDTH)
    y = np.arange(0,LENGTH)
    grid_x, grid_y = np.meshgrid(x, y)

    row_sample = 10
    col_sample = 10

    list_row = split_range(LENGTH, row_sample)
    list_col = split_range(WIDTH, col_sample)

    split_grid_y = split_box(grid_y,row_sample,col_sample)
    split_grid_x = split_box(grid_x,row_sample,col_sample)

    data_parallel = []
    for i, (ay, ax) in enumerate(zip(split_grid_y, split_grid_x)):
        # extend the search area by 5 pixels
        max_ax = max(ax.flatten()) + 5
        min_ax = min(ax.flatten()) - 5
        max_ay = max(ay.flatten()) + 5
        min_ay = min(ay.flatten()) - 5

        f0 = np.where((min_ax < xx) & (xx < max_ax) & (min_ay < yy) & (yy < max_ay))
        xx0 = xx[f0]
        yy0 = yy[f0]
        zz10 = zz1[f0]
        zz20 = zz2[f0]

        points0 = np.zeros((len(xx0),2))
        points0[:,0] = xx0
        points0[:,1] = yy0
        #print(split_grid_x[i].shape)

        data0 = (points0, zz10, zz20, ax, ay)
        data_parallel.append(data0)

    #grid_lat_all = []
    #grid_lon_all = []

    grid_lat = np.zeros((LENGTH,WIDTH), dtype=np.float32)
    grid_lon = np.zeros((LENGTH,WIDTH), dtype=np.float32)

    proNumb = inps.parallelNumb
    future = np.zeros((len(data_parallel),))
    future = list(future)
    future = parallel_process(data_parallel, function, n_jobs= proNumb, use_kwargs=False, front_num=1)

    for i in range(row_sample):
        for j in range(col_sample):
            k0 = i*col_sample + j
            kk = future[k0]
            y0 = min(list_row[i])
            y1 = max(list_row[i])
            x0 = min(list_col[j])
            x1 = max(list_col[j])
            #print(kk)
            try:
                lat0 = kk[0]
                lon0 = kk[1]
                grid_lat[y0:y1+1,x0:x1+1] = lat0
                grid_lon[y0:y1+1,x0:x1+1] = lon0
            except Exception as e:
                del e

    #grid_lat = griddata(points, zz1, (grid_x, grid_y), method='nearest')
    #grid_lon = griddata(points, zz2, (grid_x, grid_y), method='nearest')

    dataNames = get_dataNames(inps.write)
    datasetDict = dict()
    meta = readfile.read_attribute(inps.write)
    for k0 in dataNames:
        datasetDict[k0] = readfile.read(inps.write,datasetName = k0)[0]

    DEM = readfile.read(inps.write,datasetName = 'height')[0]
    grid_lat[DEM==0] = 0
    grid_lon[DEM==0] = 0
    grid_lat[grid_lat==0] = 'nan'
    grid_lon[grid_lon==0] = 'nan'
    datasetDict['latitude'] = grid_lat
    datasetDict['longitude'] = grid_lon
    write_h5(datasetDict, inps.write, metadata=meta, ref_file=None, compression=None)
    print('done.')

    return



############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Zhang Yunjun, Yuan-Kai Liu, Aug 2021             #
############################################################
# Recommend import:
#   from mintpy.utils import time_func


import numpy as np
from scipy import linalg

from mintpy.utils import ptime

MODEL_EXAMPLE = """time function configuration:
    model = {
        'polynomial' : 2,                    # int, polynomial degree with 1 (linear), 2 (quadratic), 3 (cubic), etc.
        'periodic'   : [1.0, 0.5],           # list of float, period(s) in years. 1.0 (annual), 0.5 (semiannual), etc.
        'stepDate'   : ['20061014'],         # list of str, date(s) in YYYYMMDD.
        'exp'        : {'20181026': [60],    # dict, key for onset time in YYYYMMDD(THHMM) and value for char times in integer days.
                        ...
                        },
        'log'        : {'20161231': [80],    # dict, key for onset time in YYYYMMDD(THHMM) and value for char times in integer days.
                        '20190125': [100, 200],
                        ...
                        },
        ...
    }
"""


def estimate_time_func(model, date_list, dis_ts, ref_date=None, seconds=0):
    """Deformation model estimator, using a suite of linear, periodic, step, exponential, and logarithmic function(s).

    Problem setup:
        Gm = d

    Parameters: model     - dict, time functions config, e.g. {cfg}
                date_list - list of str, dates in YYYYMMDD format
                dis_ts    - 2D np.ndarray, displacement observation in size of (num_date, num_pixel)
                ref_date  - reference date from date_list
                seconds   - float or str, acquisition time of the day info in seconds.
    Returns:    G         - 2D np.ndarray, design matrix           in size of (num_date, num_param)
                m         - 2D np.ndarray, parameter solution      in size of (num_param, num_pixel)
                e2        - 1D np.ndarray, sum of squared residual in size of (num_pixel,)
    """.format(cfg=MODEL_EXAMPLE)

    G = get_design_matrix4time_func(date_list, model, ref_date=ref_date, seconds=seconds)

    # least squares solver
    # Opt. 1: m = np.linalg.pinv(G).dot(dis_ts)
    # Opt. 2: m = scipy.linalg.lstsq(G, dis_ts, cond=1e-15)[0]
    # Numpy is not used because it can not handle NaN value in dis_ts
    m, e2 = linalg.lstsq(G, dis_ts, cond=None)[:2]

    # check empty e2 due to the rank-deficient G matrix for sigularities.
    e2 = np.array(e2)
    if e2.size == 0:
        print('\nWarning: empty e2 residues array due to a redundant or rank-deficient G matrix. This can cause sigularities.')
        print('Please check: https://docs.scipy.org/doc/scipy/reference/generated/scipy.linalg.lstsq.html#scipy.linalg.lstsq')
        print('The issue may be due to:')
        print('\t1) very small char time(s) or tau(s) of the exp/log function(s)')
        print('\t2) the onset time(s) of exp/log are far earlier than the minimum date of the time series.')
        print('Try a different char time, onset time.')
        print('Your G matrix of the temporal model: \n', G)
        raise ValueError('G matrix is redundant/rank-deficient!')

    return G, m, e2


def inps2model(inps, date_list=None, print_msg=True):
    """Convert time function inputs from namespace (inps) into dict object.

    Parameters: inps      - namespace, parsed time function inputs. E.g.:
                            Namespace(
                                date_list=['20141213', '20141225', ...]   # optional
                                polynomial=1,
                                periodic=[1.0, 0.5],
                                stepDate=['20110311', '20120928T1733'],
                                exp=[['20170910', '60', '200'], ['20171026', '200']],
                                log=[['20170910', '60', '200'], ['20171026', '200']],
                            )
                date_list - list of str, date in YYYYMMDD(THHMM) format
    Returns:    model     - dict, time function configuration, e.g. {cfg}
    """.format(cfg=MODEL_EXAMPLE)

    if not date_list:
        if hasattr(inps, 'dateList'):
            date_list = inps.date_list
        else:
            raise ValueError('date_list NOT given or found in inps!')

    dmin, dmax = date_list[0], date_list[-1]
    ymin = ptime.yyyymmdd2years(dmin)
    ymax = ptime.yyyymmdd2years(dmax)

    ## inps --> model
    model = dict()
    model['polynomial'] = inps.polynomial
    model['periodic']   = inps.periodic
    model['stepDate']   = inps.stepDate

    if inps.periodic:
        # check 1 - positive value
        if any(x <= 0 for x in inps.periodic):
            raise ValueError(f'Zero or negative input period ({inps.periodic}) found!')

    if inps.stepDate:
        # check 1 - min/max limit
        for d_step in inps.stepDate:
            if not (ymin < ptime.yyyymmdd2years(d_step) < ymax):
                raise ValueError(f'input step date ({d_step}) exceeds date limit: ({dmin} / {dmax})!')

    for func_name, strs_list in zip(['exp', 'log'], [inps.exp, inps.log]):
        func_dict = dict()

        if strs_list:
            for strs in strs_list:
                onset_time, char_times = strs[0], strs[1:]
                char_times = [float(x) for x in char_times]

                # check 1 - onset_time - str format
                date_fmt = ptime.get_date_str_format(onset_time)
                if date_fmt not in ['%Y%m%d', '%Y%m%dT%H%M']:
                    raise ValueError(f'input onset time {onset_time} is NOT in YYYYMMDD(THHMM) format!')

                # check 2 - onset_time - min/max limit
                if ptime.yyyymmdd2years(onset_time) >= ymax:
                    raise ValueError(f'input exp onset date ({onset_time}) >= the last date: {dmax}')

                # check 3 - char_time - input format
                if len(char_times) == 0:
                    msg = 'NO characteristic time inputs found!\n'
                    msg += '1+ characteristic time(s) are required for each onset date'
                    msg += f' for the {func_name} function, e.g.:\n'
                    msg += f'--{func_name} 20181026 60 OR\n'
                    msg += f'--{func_name} 20161231 80 200   # append as many char_times as you like!'
                    raise ValueError(msg)

                elif any(x <= 0 for x in char_times):
                    raise ValueError(f'Zero or negative characteristic time ({char_times}) found!')

                else:
                    int_char_times = [int(x) for x in char_times]
                    for int_char_time, char_time in zip(int_char_times, char_times):
                        if int_char_time != char_time:
                            msg = 'WARNING: float-format characteristic time detected.'
                            msg += '\nIgnore the fraction part of a day and continue: '
                            msg += f'{char_time} --> {int_char_time} days.'
                            print(msg)

                    # save as dict
                    func_dict[onset_time] = int_char_times

        # save to model
        model[func_name] = func_dict

    ## print out model summary
    if print_msg:
        print('estimate deformation model with the following assumed time functions:')
        for key, value in model.items():
            print(f'    {key:<10} : {value}')

    # warning if no polynomial found
    if 'polynomial' not in model.keys():
        raise ValueError('linear/polynomial model is NOT included! Are you sure?!')

    return model


def get_num_param(model):
    """Get the number of unknown paramters from the given time function configuration.

    Parameters: model     - dict, time functions config, e.g. {cfg}
    Returns:    num_param - int, number of unknown parameters
    """.format(cfg=MODEL_EXAMPLE)

    num_param = (
        model['polynomial'] + 1
        + len(model['periodic']) * 2
        + len(model['stepDate'])
        + sum(len(val) for key, val in model['exp'].items())
        + sum(len(val) for key, val in model['log'].items())
    )

    return num_param


#################################### Design Matrices ##########################################

def get_design_matrix4time_func(date_list, model=None, ref_date=None, seconds=0):
    """Design matrix (function model) for time functions parameter estimation.
    Parameters: date_list - list of str in YYYYMMDD format, size=num_date
                model     - dict of time functions, e.g. {cfg}
                ref_date  - reference date from date_list
                seconds   - float or str, acquisition time of the day info in seconds.
    Returns:    A         - 2D array of design matrix in size of (num_date, num_param)
                            num_param = (poly_deg + 1) + 2*len(periodic) + len(steps) + len(exp_taus) + len(log_taus)
    """.format(cfg=MODEL_EXAMPLE)

    ## prepare time info
    # convert list of date into array of years in float
    yr_diff = np.array(ptime.yyyymmdd2years(date_list, seconds=seconds))

    # reference date
    if ref_date is None:
        ref_date = date_list[0]
    yr_diff -= yr_diff[date_list.index(ref_date)]

    ## construct design matrix A
    # default model value
    if not model:
        model = {'polynomial' : 1}

    # read the models
    poly_deg   = model.get('polynomial', 0)
    periods    = model.get('periodic', [])
    steps      = model.get('stepDate', [])
    exps       = model.get('exp', dict())
    logs       = model.get('log', dict())
    num_period = len(periods)
    num_step   = len(steps)
    num_exp    = sum(len(val) for key, val in exps.items())
    num_log    = sum(len(val) for key, val in logs.items())

    num_param = (poly_deg + 1) + (2 * num_period) + num_step + num_exp + num_log
    if num_param <= 1:
        raise ValueError('NO time functions specified!')

    # initialize the design matrix
    num_date = len(yr_diff)
    A = np.zeros((num_date, num_param), dtype=np.float32)
    c0 = 0

    # update linear/polynomial term(s)
    # poly_deg of 0 --> offset
    # poly_deg of 1 --> velocity
    # ...
    c1 = c0 + poly_deg + 1
    A[:, c0:c1] = get_design_matrix4polynomial_func(yr_diff, poly_deg)
    c0 = c1

    # update periodic term(s)
    if num_period > 0:
        c1 = c0 + 2 * num_period
        A[:, c0:c1] = get_design_matrix4periodic_func(yr_diff, periods)
        c0 = c1

    # update coseismic/step term(s)
    if num_step > 0:
        c1 = c0 + num_step
        A[:, c0:c1] = get_design_matrix4step_func(date_list, steps, seconds=seconds)
        c0 = c1

    # update exponential term(s)
    if num_exp > 0:
        c1 = c0 + num_exp
        A[:, c0:c1] = get_design_matrix4exp_func(date_list, exps, seconds=seconds)
        c0 = c1

    # update logarithmic term(s)
    if num_log > 0:
        c1 = c0 + num_log
        A[:, c0:c1] = get_design_matrix4log_func(date_list, logs, seconds=seconds)
        c0 = c1

    return A



def get_design_matrix4polynomial_func(yr_diff, degree):
    """design matrix/function model of linear/polynomial velocity estimation

    d = c0 + c1 * t + 1/2 * c2 * t^2 + 1/6 * c3 * t^3 + ...

    The k! denominator makes the estimated polynomial coefficient (c_k) physically meaningful:
        k=0 makes c0 the offset;
        k=1 makes c1 the velocity;
        k=2 makes c2 the acceleration;
        k=3 makes c3 the acceleration rate;

    Parameters: yr_diff: time difference from ref_date in decimal years
                degree : polynomial models: 0=offset, 1=linear, 2=quadratic, 3=cubic, etc.
    Returns:    A      : 2D array of poly-coeff. in size of (num_date, degree+1)
    """
    A = np.zeros([len(yr_diff), degree + 1], dtype=np.float32)
    for i in range(degree+1):
        A[:,i] = (yr_diff**i) / np.math.factorial(i)

    return A


def get_design_matrix4periodic_func(yr_diff, periods):
    """design matrix/function model of periodic velocity estimation
    Parameters: yr_diff : 1D array of time difference from ref_date in decimal years
                periods : list of period in years: 1=annual, 0.5=semiannual, etc.
    Returns:    A       : 2D array of periodic sine & cosine coeff. in size of (num_date, 2*num_period)
    """
    num_date = len(yr_diff)
    num_period = len(periods)
    A = np.zeros((num_date, 2*num_period), dtype=np.float32)

    for i, period in enumerate(periods):
        c0, c1 = 2*i, 2*i+1
        A[:, c0] = np.cos(2*np.pi/period * yr_diff)
        A[:, c1] = np.sin(2*np.pi/period * yr_diff)

    return A


def get_design_matrix4step_func(date_list, step_date_list, seconds=0):
    """design matrix/function model of coseismic velocity estimation
    Parameters: date_list      : list of dates in YYYYMMDD format
                step_date_list : Heaviside step function(s) with date in YYYYMMDD
    Returns:    A              : 2D array of zeros & ones in size of (num_date, num_step)
    """
    num_date = len(date_list)
    num_step = len(step_date_list)
    A = np.zeros((num_date, num_step), dtype=np.float32)

    t = np.array(ptime.yyyymmdd2years(date_list, seconds=seconds))
    t_steps = ptime.yyyymmdd2years(step_date_list)
    for i, t_step in enumerate(t_steps):
        A[:, i] = np.array(t > t_step).flatten()

    return A


def get_design_matrix4exp_func(date_list, exp_dict, seconds=0):
    """design matrix/function model of exponential postseismic relaxation estimation

    Reference: Eq. (5) in Hetland et al. (2012, JGR).
    Note that there is a typo in the paper for this equation, based on the MInTS code, it should be:
        Sum_i{ a_i * H(t-Ti) * [1 - e^(-(t-T_i)/tau_i)] }
    instead of the one below shown in the paper:
        Sum_i{ a_i * H(t-Ti) * [1 - e^(-(t)/tau_i)] }
    where:
        a_i         amplitude      of i-th exp term
        T_i         onset time     of i-th exp term
        tau_i       char time      of i-th exp term (relaxation time)
        H(t-T_i)    Heaviside func of i-th exp term (ensuring the exp func is one-sided)

    Parameters: date_list      : list of dates in YYYYMMDD format
                exp_dict       : dict of exp func(s) info as:
                                 {{onset_time1} : [{char_time11,...,char_time1N}],
                                  {onset_time2} : [{char_time21,...,char_time2N}],
                                  ...
                                  }
                                 where onset_time is string  in YYYYMMDD format and
                                       char_time  is float32 in decimal days
    Returns:    A              : 2D array of zeros & ones in size of (num_date, num_exp)
    """
    num_date = len(date_list)
    num_exp  = sum(len(val) for key, val in exp_dict.items())
    A = np.zeros((num_date, num_exp), dtype=np.float32)

    t = np.array(ptime.yyyymmdd2years(date_list, seconds=seconds))
    # loop for onset time(s)
    i = 0
    for exp_onset in exp_dict.keys():
        # convert string to float in years
        exp_T = ptime.yyyymmdd2years(exp_onset)

        # loop for charateristic time(s)
        for exp_tau in exp_dict[exp_onset]:
            # convert time from days to years
            exp_tau /= 365.25
            A[:, i] = np.array(t > exp_T).flatten() * (1 - np.exp(-1 * (t - exp_T) / exp_tau))
            i += 1

    return A


def get_design_matrix4log_func(date_list, log_dict, seconds=0):
    """design matrix/function model of logarithmic postseismic relaxation estimation

    Reference: Eq. (4) in Hetland et al. (2012, JGR)
    Note that there is a typo in the paper for this equation, based on the MInTS code, it should be:
        Sum_i{ a_i * H(t-Ti) * [1 + log((t-T_i)/tau_i)] }
    instead of the one below shown in the paper:
        Sum_i{ a_i * H(t-Ti) * [1 + log((t)/tau_i)] }
    where:
        a_i         amplitude      of i-th log term
        T_i         onset time     of i-th log term
        tau_i       char time      of i-th log term (relaxation time)
        H(t-T_i)    Heaviside func of i-th log term (ensuring the log func is one-sided)

    Parameters: date_list      : list of dates in YYYYMMDD format
                log_dict       : dict of log func(s) info as:
                                 {{onset_time1} : [{char_time11,...,char_time1N}],
                                  {onset_time2} : [{char_time21,...,char_time2N}],
                                  ...
                                  }
                                 where onset_time is string  in YYYYMMDD format and
                                       char_time  is float32 in decimal days
    Returns:    A              : 2D array of zeros & ones in size of (num_date, num_log)
    """
    num_date = len(date_list)
    num_log  = sum(len(log_dict[x]) for x in log_dict)
    A = np.zeros((num_date, num_log), dtype=np.float32)

    t = np.array(ptime.yyyymmdd2years(date_list, seconds=seconds))
    # loop for onset time(s)
    i = 0
    for log_onset in log_dict.keys():
        # convert string to float in years
        log_T = ptime.yyyymmdd2years(log_onset)

        # loop for charateristic time(s)
        for log_tau in log_dict[log_onset]:
            # convert time from days to years
            log_tau /= 365.25

            olderr = np.seterr(invalid='ignore', divide='ignore')
            A[:, i] = np.array(t > log_T).flatten() * np.nan_to_num(np.log(1 + (t - log_T) / log_tau), nan=0, neginf=0)
            np.seterr(**olderr)
            i += 1

    return A



############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Zhang Yunjun, Heresh Fattahi, 2013               #
############################################################


import os

import numpy as np

from mintpy import view
from mintpy.objects import HDFEOS
from mintpy.utils import ptime, readfile, utils as ut, writefile


##############################################################################
def read_data(inps):
    # metadata
    atr = readfile.read_attribute(inps.file)

    if 'WAVELENGTH' in atr.keys():
        range2phase = -4 * np.pi / float(atr['WAVELENGTH'])

    # change reference pixel
    if inps.ref_lalo:
        if 'Y_FIRST' in atr.keys():
            coord = ut.coordinate(atr)
            ref_y, ref_x = coord.geo2radar(inps.ref_lalo[0], inps.ref_lalo[1])[0:2]
            inps.ref_yx = [ref_y, ref_x]
        else:
            raise ValueError("input file is not geocoded --> reference point in lat/lon is NOT support")

    if inps.ref_yx:
        atr['REF_Y'] = inps.ref_yx[0]
        atr['REF_X'] = inps.ref_yx[1]
        if 'Y_FIRST' in atr.keys():
            coord = ut.coordinate(atr)
            ref_lat, ref_lon = coord.radar2geo(inps.ref_yx[0], inps.ref_yx[1])[0:2]
            atr['REF_LAT'] = ref_lat
            atr['REF_LON'] = ref_lon
        print(f'change reference point to y/x: {inps.ref_yx}')

    # various file types
    print(f'read {inps.dset} from file {inps.file}')
    k = atr['FILE_TYPE']
    if k == 'velocity':
        # read/prepare data
        if not inps.dset:
            inps.dset = 'velocity'
            print('No selected datset, assuming "velocity" and continue.')
        data, atr = readfile.read(inps.file, datasetName=inps.dset)

        # convert velocity to cumulative displacement
        if inps.dset == 'velocity':
            print('convert velocity to displacement for {}'.format(atr['DATE12']))
            dt1, dt2 = ptime.date_list2vector(atr['DATE12'].split('_'))[0]
            data *= (dt2 - dt1).days / 365.25

        # convert data from the unit of meter to radian
        if atr.get('UNIT', 'm/year').startswith('m'):
            print('convert the unit from meter to radian')
            data *= range2phase
            atr['UNIT'] = 'radian'

        # apply the custom spatial referencing
        if inps.ref_yx:
            data -= data[inps.ref_yx[0], inps.ref_yx[1]]

        # metadata
        atr['FILE_TYPE'] = '.unw'

        # output filename
        if not inps.outfile:
            inps.outfile = os.path.join(os.path.dirname(inps.file), '{}.unw'.format(atr['DATE12']))

    elif k == 'timeseries':
        # date1 and date2
        if '_' in inps.dset:
            date1, date2 = ptime.yyyymmdd(inps.dset.split('_'))
        else:
            date1 = atr['REF_DATE']
            date2 = ptime.yyyymmdd(inps.dset)

        # read/prepare data
        data = readfile.read(inps.file, datasetName=date2)[0]
        data -= readfile.read(inps.file, datasetName=date1)[0]
        print('converting range to phase')
        data *= range2phase
        if inps.ref_yx:
            data -= data[inps.ref_yx[0], inps.ref_yx[1]]

        # metadata
        atr['DATE'] = date1[2:8]
        atr['DATE12'] = f'{date1[2:8]}-{date2[2:8]}'
        atr['FILE_TYPE'] = '.unw'
        atr['UNIT'] = 'radian'

        # output filename
        if not inps.outfile:
            inps.outfile = f'{date1}_{date2}.unw'
            if inps.file.startswith('geo_'):
                inps.outfile = 'geo_'+inps.outfile

    elif k == 'HDFEOS':
        dname = inps.dset.split('-')[0]

        # date1 and date2
        if dname == 'displacement':
            if '-' in inps.dset:
                suffix = inps.dset.split('-')[1]
                if '_' in suffix:
                    date1, date2 = ptime.yyyymmdd(suffix.split('_'))
                else:
                    date1 = atr['REF_DATE']
                    date2 = ptime.yyyymmdd(suffix)
            else:
                raise ValueError(f"No '-' in input dataset! It is required for {dname}")
        else:
            date_list = HDFEOS(inps.file).get_date_list()
            date1 = date_list[0]
            date2 = date_list[-1]
        date12 = f'{date1}_{date2}'

        # read / prepare data
        slice_list = readfile.get_slice_list(inps.file)
        if 'displacement' in inps.dset:
            # read/prepare data
            slice_name1 = view.search_dataset_input(slice_list, f'{dname}-{date1}')[0][0]
            slice_name2 = view.search_dataset_input(slice_list, f'{dname}-{date2}')[0][0]
            data = readfile.read(inps.file, datasetName=slice_name1)[0]
            data -= readfile.read(inps.file, datasetName=slice_name2)[0]
            print('converting range to phase')
            data *= range2phase
            if inps.ref_yx:
                data -= data[inps.ref_yx[0], inps.ref_yx[1]]
        else:
            slice_name = view.search_dataset_input(slice_list, inps.dset)[0][0]
            data = readfile.read(inps.file, datasetName=slice_name)[0]

        # metadata
        atr['DATE'] = date1[2:8]
        atr['DATE12'] = f'{date1[2:8]}-{date2[2:8]}'
        if dname == 'displacement':
            atr['FILE_TYPE'] = '.unw'
            atr['UNIT'] = 'radian'
        elif 'coherence' in dname.lower():
            atr['FILE_TYPE'] = '.cor'
            atr['UNIT'] = '1'
        elif dname == 'height':
            atr['FILE_TYPE'] = '.dem'
            atr['DATA_TYPE'] = 'int16'
        else:
            raise ValueError(f'unrecognized input dataset type: {inps.dset}')

        # output filename
        if not inps.outfile:
            inps.outfile = '{}{}'.format(date12, atr['FILE_TYPE'])

    elif k == 'ifgramStack':
        dname, date12 = inps.dset.split('-')
        date1, date2 = date12.split('_')

        # read / prepare data
        data = readfile.read(inps.file, datasetName=inps.dset)[0]
        if dname.startswith('unwrapPhase'):
            if 'REF_X' in atr.keys():
                data -= data[int(atr['REF_Y']), int(atr['REF_X'])]
                print('consider reference pixel in y/x: ({}, {})'.format(atr['REF_Y'], atr['REF_X']))
            else:
                print('No REF_Y/X found.')

        # metadata
        atr['DATE'] = date1[2:8]
        atr['DATE12'] = f'{date1[2:8]}-{date2[2:8]}'
        if dname.startswith('unwrapPhase'):
            atr['FILE_TYPE'] = '.unw'
            atr['UNIT'] = 'radian'
        elif dname == 'coherence':
            atr['FILE_TYPE'] = '.cor'
            atr['UNIT'] = '1'
        elif dname == 'wrapPhase':
            atr['FILE_TYPE'] = '.int'
            atr['UNIT'] = 'radian'
        elif dname == 'connectComponent':
            atr['FILE_TYPE'] = '.conncomp'
            atr['UNIT'] = '1'
            atr['DATA_TYPE'] = 'byte'
        else:
            raise ValueError(f'unrecognized dataset type: {inps.dset}')

        # output filename
        if not inps.outfile:
            inps.outfile = '{}{}'.format(date12, atr['FILE_TYPE'])
            if inps.file.startswith('geo_'):
                inps.outfile = 'geo_'+inps.outfile

    else:
        # read data
        data = readfile.read(inps.file, datasetName=inps.dset)[0]

        if inps.outfile:
            # get file extension
            fbase, fext = os.path.splitext(inps.outfile)
            # ignore certain meaningless file extensions
            while fext in ['.geo', '.rdr', '.full', '.wgs84']:
                fbase, fext = os.path.splitext(fbase)
            if not fext:
                fext = os.path.basename(fbase)

            atr['FILE_TYPE'] = fext
        else:
            # metadata
            if 'coherence' in k.lower():
                atr['FILE_TYPE'] = '.cor'
            elif k in ['mask']:
                atr['FILE_TYPE'] = '.msk'
            elif k in ['geometry'] and inps.dset == 'height':
                if 'Y_FIRST' in atr.keys():
                    atr['FILE_TYPE'] = '.dem'
                else:
                    atr['FILE_TYPE'] = '.hgt'
                atr['UNIT'] = 'm'
            else:
                atr['FILE_TYPE'] = '.unw'

            inps.outfile = '{}{}'.format(os.path.splitext(inps.file)[0], atr['FILE_TYPE'])

    # mask
    if inps.mask_file:
        for m_file in inps.mask_file:
            print(f'mask data based on input file: {m_file}')
            mask = readfile.read(m_file)[0]
            mask *= ~np.isnan(data)
            data[mask==0] = np.nan

    # get rid of starting . if output as hdf5 file
    if inps.outfile.endswith('.h5'):
        if atr['FILE_TYPE'].startswith('.'):
            atr['FILE_TYPE'] = atr['FILE_TYPE'][1:]

    atr['PROCESSOR'] = 'roipac'
    return data, atr, inps.outfile


def clean_metadata4roipac(atr_in):
    atr = {}
    for key, value in atr_in.items():
        atr[key] = str(value)

    # drop the following keys
    key_list = ['width', 'Width', 'samples', 'length', 'lines',
                'SUBSET_XMIN','SUBSET_XMAX','SUBSET_YMIN','SUBSET_YMAX',
               ]
    for key in key_list:
        if key in atr.keys():
            atr.pop(key)

    # drop all keys that are not all UPPER_CASE
    key_list = list(atr.keys())
    for key in key_list:
        if not key.isupper():
            atr.pop(key)

    atr['FILE_LENGTH'] = atr['LENGTH']
    return atr


##############################################################################
def save_roipac(inps):

    # read data and metadata
    data, atr, out_file = read_data(inps)

    # remove non-roipac metadata
    if not inps.keepAllMetadata:
        atr = clean_metadata4roipac(atr)

    # write
    writefile.write(data, out_file=out_file, metadata=atr)

    return



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Zhang Yunjun, Aug 2022        #
############################################################


import os
import sys

from mintpy.utils.arg_utils import create_argument_parser

##############################################################################
REFERENCE = """references:
  Bagnardi, M., and A. Hooper (2018), Inversion of Surface Deformation Data for Rapid Estimates of
    Source Parameters and Uncertainties: A Bayesian Approach, Geochemistry, Geophysics, Geosystems,
    19, doi:10.1029/2018GC007585.
  Yunjun, Z., Amelung, F., & Aoki, Y. (2021), Imaging the hydrothermal system of Kirishima volcanic
    complex with L-band InSAR time series, Geophysical Research Letters, 48(11), e2021GL092879.
    doi:10.1029/2021GL092879
"""

EXAMPLE = """example:
  save_gbis.py velocity.h5 -g inputs/geometryGeo.h5 -o AlosDT73_20081012_20100302.mat
  save_gbis.py 20150223_20161031_msk.unw -g inputs/geometryGeo.h5 -o Alos2DT23_20150223_20161031.mat
  save_gbis.py 20150223_20161031_msk.unw -g inputs/geometryGeo.h5 --out-dir ../Model/data --ellipsoid2geoid

  #for non-veloicty dataset, e.g. step or annualAmplitude
  save_gbis.py velocity.h5 step20080221    -o AlosDT73_step20080221.mat
  save_gbis.py velocity.h5 annualAmplitude -o AlosDT73_annualAmplitude.mat
"""

def create_parser(subparsers=None):
    synopsis = 'Convert MintPy product to GBIS .mat format.'
    epilog = REFERENCE + '\n' + EXAMPLE
    name = __name__.split('.')[-1]
    parser = create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('file', help='deformation file.')
    parser.add_argument('dset', nargs='?',
                        help='date/date12 of timeseries, or date12 of interferograms to be converted')
    parser.add_argument('-g','--geometry', dest='geom_file', required=True, help='geometry file')
    parser.add_argument('-m', '--mask', dest='mask_file', help='mask file.')

    parser.add_argument('--ref-lalo', dest='ref_lalo', type=float, nargs=2,
                        help='custom reference pixel in lat/lon')
    parser.add_argument('--nodisplay', dest='disp_fig', action='store_false',
                        help='do not display the figure')
    parser.add_argument('-o', '--output', dest='outfile', help='output file name.')
    parser.add_argument('--out-dir', dest='outdir',
                        help='custom output directory, ONLY IF --output is not specified.')
    parser.add_argument('--ellipsoid2geoid', action='store_true',
                        help='Convert the height of ellipsoid to geoid using "geoidheight" module\n'+
                             'Download & install geoidheight as below:\n'+
                             'https://github.com/geodesymiami/2021_Kirishima')
    return parser


def cmd_line_parse(iargs=None):
    parser = create_parser()
    inps = parser.parse_args(args=iargs)

    inps.argv = iargs if iargs else sys.argv[1:]
    print('{} {}'.format(os.path.basename(__file__), ' '.join(inps.argv)))

    return inps


##############################################################################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.save_gbis import save_gbis

    # run
    save_gbis(inps)


##############################################################################
if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Zhang Yunjun, Feb 2020                           #
############################################################
# Useful links:
#   IGS (NASA): https://cddis.nasa.gov/Data_and_Derived_Products/GNSS/atmospheric_products.html
#   IMPC (DLR): https://impc.dlr.de/products/total-electron-content/near-real-time-tec/nrt-tec-global/
# Recommend usage:
#   from mintpy.simulation import iono


import datetime as dt
import os
import re

import matplotlib.pyplot as plt
import numpy as np
from scipy import interpolate

from mintpy.utils import readfile, utils0 as ut

# constants
SPEED_OF_LIGHT = 299792458 # m/s
EARTH_RADIUS = 6371e3      # Earth radius in meters
K = 40.31                  # m^3/s^2, constant

SAR_BAND = {
    'L' : 1.257e9,  # NISAR-L, ALOS-2
    'S' : 3.2e9,    # NISAR-S
    'C' : 5.405e9,  # Sentinel-1
    'X' : 9.65e9,   # TerraSAR-X
    'Ka': 35.75e9,  # SWOT
}


######################## Ionospheric Mapping Functions #########################
# Reference:
#   Yunjun, Z., Fattahi, H., Pi, X., Rosen, P., Simons, M., Agram, P., & Aoki, Y. (2022). Range
#     Geolocation Accuracy of C-/L-band SAR and its Implications for Operational Stack Coregistration.
#     IEEE Trans. Geosci. Remote Sens., 60, doi:10.1109/TGRS.2022.3168509.

def vtec2range_delay(vtec, inc_angle, freq, obs_type='phase'):
    """Calculate/predict the range delay in SAR from TEC in zenith direction.

    Equation (6-11) from Yunjun et al. (2022).

    Parameters: vtec      - float, zenith TEC in TECU
                inc_angle - float/np.ndarray, incidence angle at the ionospheric shell in deg
                freq      - float, radar carrier frequency in Hz.
                obs_type  - str, given the same iono, the impact on offset (amplitude) and phase is reversed.
    Returns:    rg_delay  - float/np.ndarray, predicted range delay in meters
    """
    # ignore no-data value in inc_angle
    if isinstance(inc_angle, np.ndarray):
        inc_angle[inc_angle == 0] = np.nan

    # convert to TEC in LOS based on equation (3) in Chen and Zebker (2012)
    ref_angle = iono_incidence2refraction_angle(inc_angle, vtec, freq)
    tec = vtec / np.cos(ref_angle * np.pi / 180.0)

    # calculate range delay based on equation (1) in Chen and Zebker (2012)
    range_delay = (tec * 1e16 * K / (freq**2)).astype(np.float32)

    # group delay = phase advance * -1
    if obs_type != 'phase':
        range_delay *= -1.

    return range_delay


def iono_incidence2refraction_angle(inc_angle, vtec, freq):
    """Calculate the refraction angle for the ionospheric shell.

    Reference:
        Equation (8) in Yunjun et al. (2022, TGRS)
        Equation (26) in Bohm & Schuh (2013) Chapter 2.

    Parameters: inc_angle - float / 1/2D np.ndarray, incidence angle in deg
                vtec      - float / 1D np.ndarray, zenith TEC in TECU
                freq      - float, radar carrier frequency in Hz.
    Returns:    ref_angle - float / 1/2/3D np.ndarray, refraction angle in deg
    """
    if isinstance(vtec, np.ndarray):
        # only 1D array is supported
        if vtec.ndim > 1:
            raise ValueError(f'input vtec dimension ({vtec.ndim}) > 1!')

        # tile to ensure same shape between vtec and inc_angle
        if isinstance(inc_angle, np.ndarray) and inc_angle.shape != vtec.shape:
            num_vtec = vtec.size
            if inc_angle.ndim == 1:
                vtec = np.tile(vtec.reshape(-1, 1), (1, inc_angle.size))
                inc_angle = np.tile(inc_angle.reshape(1, -1), (num_vtec, 1))
            elif inc_angle.ndim == 2:
                vtec = np.tile(vtec.reshape(-1, 1, 1), (1, inc_angle.shape[0], inc_angle.shape[1]))
                inc_angle = np.tile(inc_angle[np.newaxis, :, :], (num_vtec, 1, 1))
            else:
                raise ValueError(f'input inc_angle dimension ({inc_angle.ndim}) > 2!')

    # Equation (26) in Bohm & Schuh (2013) Chapter 2.
    n_iono_group = 1 + K * vtec * 1e16 / freq**2

    # Equation (8) in Yunjun et al. (2022, TGRS)
    ref_angle = np.arcsin(1 / n_iono_group * np.sin(inc_angle * np.pi / 180)) * 180 / np.pi

    return ref_angle


def prep_geometry_iono(geom_file, box=None, iono_height=450e3, print_msg=True):
    """Prepare geometry info of LOS vector at thin-shell ionosphere.

    Equation (11-12) in Yunjun et al. (2022, TGRS)

    Parameters: geom_file      - str, path to the geometry file in HDF5/MintPy format
                box            - tuple of 4 int, box of interest in (x0, y0, x1, y1)
                iono_height    - float, height of the assume effective thin-shell ionosphere in m
    Returns:    iono_inc_angle - 2D np.ndarray / float, incidence angle in degree
                iono_lat/lon   - float, latitude/longitude of LOS vector at the thin-shell in deg
                iono_height    - float, height of the assume effective thin-shell ionosphere in m
    """

    def get_center_lat_lon(geom_file, box=None):
        """Get the lat/lon of the scene center"""
        meta = readfile.read_attribute(geom_file)
        if box is None:
            box = (0, 0, int(meta['WIDTH']), int(meta['LENGTH']))

        col_c = int((box[0] + box[2]) / 2)
        row_c = int((box[1] + box[3]) / 2)
        if 'Y_FIRST' in meta.keys():
            lat0 = float(meta['Y_FIRST'])
            lon0 = float(meta['X_FIRST'])
            lat_step = float(meta['Y_STEP'])
            lon_step = float(meta['X_STEP'])
            lat_c = lat0 + lat_step * row_c
            lon_c = lon0 + lon_step * col_c
        else:
            box_c = (col_c, row_c, col_c+1, row_c+1)
            lat_c = float(readfile.read(geom_file, datasetName='latitude',  box=box_c)[0])
            lon_c = float(readfile.read(geom_file, datasetName='longitude', box=box_c)[0])
        return lat_c, lon_c

    # inc_angle on the ground
    inc_angle = readfile.read(geom_file, datasetName='incidenceAngle', box=box)[0]
    inc_angle = np.squeeze(inc_angle)
    inc_angle[inc_angle == 0] = np.nan
    inc_angle_center = np.nanmean(inc_angle)
    if print_msg:
        print('incidence angle on the ground     min/max: {:.1f}/{:.1f} deg'.format(np.nanmin(inc_angle),
                                                                                    np.nanmax(inc_angle)))

    # inc_angle on the thin-shell ionosphere - equation (11)
    iono_inc_angle = incidence_angle_ground2iono(inc_angle, iono_height=iono_height)
    if print_msg:
        print('incidence angle on the ionosphere min/max: {:.1f}/{:.1f} deg'.format(np.nanmin(iono_inc_angle),
                                                                                    np.nanmax(iono_inc_angle)))

    # center lat/lon on the ground & thin-shell ionosphere - equation (12)
    lat, lon = get_center_lat_lon(geom_file, box=box)
    az_angle = readfile.read(geom_file, datasetName='azimuthAngle', box=box)[0]
    az_angle[az_angle == 0] = np.nan
    az_angle_center = np.nanmean(az_angle)
    iono_lat, iono_lon = lalo_ground2iono(lat, lon, inc_angle=inc_angle_center, az_angle=az_angle_center)

    if print_msg:
        print(f'center lat/lon  on the ground    : {lat:.4f}/{lon:.4f} deg')
        print(f'center lat/lon  on the ionosphere: {iono_lat:.4f}/{iono_lon:.4f} deg')

    return iono_inc_angle, iono_lat, iono_lon, iono_height


def incidence_angle_ground2iono(inc_angle, iono_height=450e3):
    """Calibrate the incidence angle of LOS vector on the ground surface to the ionosphere shell.

    Equation (11) in Yunjun et al. (2022, TGRS)

    Parameters: inc_angle      - float/np.ndarray, incidence angle on the ground in degrees
                iono_height    - float, effective ionosphere height in meters
                                 under the thin-shell assumption
    Returns:    inc_angle_iono - float/np.ndarray, incidence angle on the iono shell in degrees
    """
    # ignore nodata in inc_angle
    if isinstance(inc_angle, np.ndarray):
        inc_angle[inc_angle == 0] = np.nan

    # deg -> rad & copy to avoid changing input variable
    inc_angle = np.array(inc_angle) * np.pi / 180

    # calculation
    inc_angle_iono = np.arcsin(EARTH_RADIUS * np.sin(inc_angle) / (EARTH_RADIUS + iono_height))
    inc_angle_iono *= 180. / np.pi

    return inc_angle_iono


def lalo_ground2iono(lat, lon, inc_angle, az_angle=None, head_angle=None, iono_height=450e3, method='spherical_distance'):
    """Calculate lat/lon of IPP with the given lat/lon on the ground and LOS geometry

    Equation (12) in Yunjun et al. (2021, TGRS).

    Parameters: lat/lon      - float, latitude/longitude of the point on the ground in degrees
                inc_angle    - float/np.ndarray, incidence angle of the line-of-sight vector on the ground in degrees
                az_angle     - float/np.ndarray, azimuth   angle of the line-of-sight vector
                head_angle   - float, heading angle of the satellite orbit in degrees
                               from the north direction with positive in clockwise direction
                iono_height  - float, height of the ionosphere thin-shell in meters
    Returns:    lat/lon_iono - float, latitude/longitude of the point on the thin-shell ionosphere in degrees
    """
    # LOS incidence angle at ionospheric level
    inc_angle_iono = incidence_angle_ground2iono(inc_angle)

    # geocentric angle distance between the SAR pixel and IPP
    dist_angle = inc_angle - inc_angle_iono

    # LOS azimuth angle (from the ground to the satellite)
    # measured from the north with anti-clockwise as positive (ISCE-2 convention).
    if az_angle is None:
        # heading angle -> azimuth angle
        if head_angle is not None:
            az_angle = ut.heading2azimuth_angle(head_angle)
    if az_angle is None:
        raise ValueError('az_angle can not be None!')

    # option 1 - spherical_distance
    # link:
    #   https://gis.stackexchange.com/questions/5821 [there is a typo there]
    #   http://www.movable-type.co.uk/scripts/latlong.html [used]
    if method == 'spherical_distance':
        # deg -> rad & copy to avoid changing input variable
        lat = np.array(lat) * np.pi / 180.
        lon = np.array(lon) * np.pi / 180.
        dist_angle *= np.pi / 180.
        az_angle *= np.pi / 180.

        # calculate
        lat_iono = np.arcsin(  np.sin(lat) * np.cos(dist_angle)
                             + np.cos(lat) * np.sin(dist_angle) * np.cos(az_angle))
        dlon = np.arctan2(np.sin(dist_angle) * np.cos(lat) * np.sin(az_angle) * -1.,
                          np.cos(dist_angle) - np.sin(lat) * np.sin(lat_iono))
        lon_iono = np.mod(lon + dlon + np.pi, 2. * np.pi) - np.pi

        # rad -> deg
        lat_iono *= 180. / np.pi
        lon_iono *= 180. / np.pi

    # option 2 - pyproj
    # link: https://pyproj4.github.io/pyproj/stable/api/geod.html#pyproj.Geod.fwd
    elif method == 'pyproj':
        import pyproj
        geod = pyproj.Geod(ellps='WGS84')
        dist = dist_angle * (np.pi / 180.) * EARTH_RADIUS
        lon_iono, lat_iono = geod.fwd(lon, lat, az=-az_angle, dist=dist)[:2]

    else:
        raise ValueError(f'Un-recognized method: {method}')

    ## obsolete
    ## offset angle from equation (25) in Chen and Zebker (2012)
    ## ~3 degree in magnitude for Sentinel-1 asc track
    #off_iono = inc_angle - np.arcsin(EARTH_RADIUS / (EARTH_RADIUS + iono_height) * np.sin(np.pi - inc_angle))
    #lat += off_iono * np.cos(head_angle) * 180. / np.pi
    #lon += off_iono * np.sin(head_angle) * 180. / np.pi

    return lat_iono, lon_iono



############################ JPL high resolution GIM ###########################
# JPL high resolution GIM
#   Spatial  resolution in latitude / longitude [deg]: 1.0 / 1.0
#   Temporal resolution [min]: 15

def get_gim_tec_list(gim_tec_dir, dt_objs, iono_lat, iono_lon):
    """Get the GIM TEC value at loc (lat/lon) of interest.

    Parameters: gim_tec_dir - str, path to the GIM_TEC directory
                dt_objs     - list of datetime.datetime objects
                iono_lat    - float, latitude  of LOS vector at ionosphere in degree
                iono_lon    - float, longitude of LOS vector at ionosphere in degree
    Returns:    vtec        - 1D np.ndarray of zenith TEC in TECU
    """

    num_date = len(dt_objs)
    vtec = np.zeros(num_date, dtype=np.float32) * np.nan

    for i, dt_obj in enumerate(dt_objs):
        # get gim tec file name
        m_step = 15
        h0 = dt_obj.hour
        m0 = int(np.floor(dt_obj.minute / m_step) * m_step)
        m1 = m0 + m_step
        if m1 < 60:
            h1 = h0
        else:
            m1 = 0
            h1 = h0 + 1
        hm0 = f'{h0:02d}{m0:02d}'
        hm1 = f'{h1:02d}{m1:02d}'
        tec_file = 'gim_{}_{}_{}.tecgrd.txt'.format(dt_obj.strftime('%y%m%d'), hm0, hm1)
        tec_file = os.path.join(gim_tec_dir, tec_file)

        # read tec file
        if os.path.isfile(tec_file):
            lats, lons, vtec_mat = read_gim_tec_grid_file(tec_file)

            # value of the nearest grid
            ind_lat = np.argmin(np.abs(lats - iono_lat))
            ind_lon = np.argmin(np.abs(lons - iono_lon))
            vtec[i] = vtec_mat[ind_lat, ind_lon]
        else:
            print(f'WARNING: NO file found in {tec_file}. Set to NaN.')

    return vtec


def read_gim_tec_grid_file(tec_file, display=False):
    """Read JPL high resolution GIM grid file.

    Parameters: tec_file - str, path of the gim_*.tecgrd.txt file
    Returns:    lat/lon  - 1D np.ndarray in size of 181/361 in degrees
                tec      - 2D np.ndarray in size of (181, 361) in TECU
    """
    lat = np.loadtxt(tec_file, skiprows=2, max_rows=1)
    lon = np.loadtxt(tec_file, skiprows=3, max_rows=1)
    tec = np.loadtxt(tec_file, skiprows=4)

    # plot
    if display:
        fig, ax = plt.subplots(nrows=1, ncols=1, figsize=[10, 4])
        im = ax.imshow(tec, extent=(-180.5, 180.5, -90.5, 90.5))
        # colorbar
        cbar = fig.colorbar(im)
        cbar.set_label('zenith TEC [TECU]')
        # axis format
        ax.set_xlabel('longitude [deg]')
        ax.set_ylabel('latitude [deg]')
        fig.tight_layout()
        # output
        plt.show()

    return lat, lon, tec


def get_sub_tec_list(gim_tec_dir, date_list, iono_lat, iono_lon,
                     interp_method='nearest', fill_value=np.nan):
    """Interpolate / extract SUB/GIM TEC value at loc (lat/lon) of interest.

    Parameters: gim_tec_dir   - str, path of GIM TEC data directorry
                date_list     - list of str, dates in YYYYMMDD format
                iono_lat/lon  - float, latitude / longitude in degree of TOP TEC of interest
                interp_method - str, interpolation method
                fill_value    - float32, the filling value for missing dates
    Returns:    date_list     - list of str, dates in YYYYMMDD format
                tec_ipp       - 1D np.ndarray in float, total       TEC at IPP in TECU
                tec_top_tpp   - 1D np.ndarray in float, topside     TEC at TPP in TECU
                tec_sub_ipp   - 1D np.ndarray in float, sub-orbital TEC at IPP in TECU
                tec_sub_tpp   - 1D np.ndarray in float, sub-orbital TEC at TPP in TECU
                tDicts        - list of dict, each dict contains all info of GIM TEC file
    """
    num_date = len(date_list)
    tec_ipp     = np.zeros(num_date, dtype=np.float32) * fill_value
    tec_top_tpp = np.zeros(num_date, dtype=np.float32) * fill_value
    tec_sub_ipp = np.zeros(num_date, dtype=np.float32) * fill_value
    tec_sub_tpp = np.zeros(num_date, dtype=np.float32) * fill_value

    # read TEC into tDicts (list of dict objects)
    tDicts = []
    for date_str in date_list:
        tec_file = os.path.join(gim_tec_dir, f'subtec_sent1_{date_str}.txt.dt')
        if os.path.isfile(tec_file):
            tDict = read_sub_tec(tec_file, print_msg=False)
        else:
            tDict = None
        tDicts.append(tDict)

    # interpolate TOP TEC at lat/lon of interest
    # tDicts --> gim/top/sub_tec
    for i in range(num_date):
        tDict = tDicts[i]
        if tDict is None:
            continue

        if interp_method == 'nearest':
            ratio_lon2lat = np.cos(iono_lat * np.pi / 180.)
            dist_deg = ( (tDict['lat'] - iono_lat) ** 2 +
                        ((tDict['lon'] - iono_lon) * ratio_lon2lat) ** 2 ) ** 0.5
            ind = np.argmin(dist_deg)
            tec_ipp[i]     = tDict['tec_ipp'][ind]
            tec_top_tpp[i] = tDict['tec_top_tpp'][ind]
            tec_sub_ipp[i] = tDict['tec_sub_ipp'][ind]
            tec_sub_tpp[i] = tDict['tec_sub_tpp'][ind]

        elif interp_method == 'nearestLat':
            ind = np.argmin(np.abs(tDict['lat'] - iono_lat))
            tec_ipp[i]     = tDict['tec_ipp'][ind]
            tec_top_tpp[i] = tDict['tec_top_tpp'][ind]
            tec_sub_ipp[i] = tDict['tec_sub_ipp'][ind]
            tec_sub_tpp[i] = tDict['tec_sub_tpp'][ind]

        elif interp_method == 'linear':
            # NOTE by Yunjun at 2021-08-12: extrapolate give abnormal large value, thus is not good.
            for d_tec, key in zip([ tec_ipp,   tec_top_tpp,   tec_sub_ipp,   tec_sub_tpp ],
                                  ['tec_ipp', 'tec_top_tpp', 'tec_sub_ipp', 'tec_sub_tpp']):
                if tDict['lat'].size <= 2:
                    # use nearest instead
                    ind = np.argmin(np.abs(tDict['lat'] - iono_lat))
                    d_tec[i] = tDict[key][ind]

                else:
                    func = interpolate.interp1d(tDict['lat'], tDict[key],
                                                kind=interp_method,
                                                fill_value='extrapolate',
                                                assume_sorted=False)
                    d_tec[i] = func(iono_lat)

        elif interp_method == 'mean':
            tec_ipp[i]     = np.mean(tDict['tec_ipp'])
            tec_top_tpp[i] = np.mean(tDict['tec_top_tpp'])
            tec_sub_ipp[i] = np.mean(tDict['tec_sub_ipp'])
            tec_sub_tpp[i] = np.mean(tDict['tec_sub_tpp'])

        elif interp_method == 'median':
            tec_ipp[i]     = np.median(tDict['tec_ipp'])
            tec_top_tpp[i] = np.median(tDict['tec_top_tpp'])
            tec_sub_ipp[i] = np.median(tDict['tec_sub_ipp'])
            tec_sub_tpp[i] = np.median(tDict['tec_sub_tpp'])

    return date_list, tec_ipp, tec_top_tpp, tec_sub_ipp, tec_sub_tpp, tDicts


def read_sub_tec(tec_file, version=2.1, print_msg=True):
    """Read the JPL SUB/GIM TEC file.

    Parameters: tec_file - str, path of the sub TEC text file
    Returns:    tDict    - dict, dictionary of GIM/SUB/TOP_TEC data and time/location info

    Acronyms:
        TGT: target  piercing point, where a shell at 450   km is assumed, previously known as IPP.
        TPP: topside piercing point, where a shell at 1,800 km is assumed.
        LEO: low earth orbit, where the SAR satellite is (~700 km).
        LOS: line-of-sight

    Version 2. Format of the data file provided by Xiaoqing Xi in August 6, 2020.
    COL01:  GPSSEC      (GPS seconds)
    COL02:  UThrs       (UTC in hours)
    COL03:  LThrs       (local solar time in hours)
    COL04:  SVN         (space vehicle number)
    COL05:  AZMdeg      (observation azimuth   angle)
    COL06:  ELVdeg      (observation elevation angle)
    COL07:  TPPLATdeg   (TPP latitude)             [*]
    COL08:  TPPLONdeg   (TPP longitude)            [*]
    COL09:  LEOALTkm    (LEO altitude  in km)
    COL10:  LEOLATdeg   (LEO latitude  in deg)
    COL11:  LEOLONdeg   (LEO longitude in deg)
    COL12:  TGTLATdeg   (target latitude)
    COL13:  TGTLONdeg   (target longitude)
    COL14:  STECRMB     (slant TEC, bias removed)
    COL15:  TOPTEC_TPP  (topTEC  at TPP)           [*]
    COL16:  GIMTEC_TPP  (GIM TEC at TPP)
    COL17:  SUBTEC_TPP  (subTEC  at TPP)
    COL18:  GIMTEC_TGT  (GIM TEC at TGT at 450 km) [*]
    COL19:  GIMTEC_LEO  (GIM TEC at the projected LEO coordinates)

    Version 1. Format of the data file provided by Xiaoqing Xi in July 9, 2020.
    COL01: GPSSEC,  GPS seconds past J2000
    COL02: UTC      [sec] within the day
    COL03: LThrs    [hours], local solar time
    COL04: SVN      [integer], GPS space vehicle number
    COL05: AZM      [deg], observation azimuth angle
    COL06: ELV      [deg], observation elevation angle (90 – zenith)
    COL07: SUBLAT   [deg], IPP latitude              [*]
    COL08: SUBLON   [deg], IPP longitude             [*]
    COL09: LEOALT   [km],  LEO altitude
    COL10: LEOLAT   [deg], LEO latitude
    COL11: LEOLON   [deg], LEO longitude
    COL12: STECRMB  [TECU], line-of-sight slant  TEC
    COL13: TOPTEC   [TECU], topside     vertical TEC [*]
    COL14: GIMTRACK [TECU], GIM         vertical TEC [*]
    COL15: SUBTEC   [TECU], sub-orbital vertical TEC [*]
    """

    if not os.path.isfile(tec_file):
        raise FileNotFoundError(f'No file found in {tec_file}')

    if print_msg:
        print(f'read JPL GIM TEC data from file: {tec_file}')

    # read data from text file
    fc = np.loadtxt(tec_file, dtype=bytes).astype(float)
    if fc.ndim == 1:
        fc = fc.reshape(1, -1)

    # read lat/lon of the (topside) ionospheric pierce point (TPP / IPP)
    (lat, lon) = fc[:, 6:8].T
    lon = ut.wrap(lon, wrap_range=(-180, 180))

    # read the topside, sub-orbital and total vertical TEC
    if version == 2.1:
        # version 2.1: GIMTEC_TGT, TOPTEC_TPP and (GIMTEC_TGT - TOPTEC_TPP)
        #tec_tpp     = fc[:, 15].T
        tec_top_tpp = fc[:, 14].T
        tec_sub_tpp = fc[:, 16].T
        tec_ipp     = fc[:, 17].T
        tec_sub_ipp = tec_ipp - tec_top_tpp

    elif version == 2.0:
        # version 2.0: GIMTEC_TPP, TOPTEC_TPP and SUBTEC_TPP
        (TOPTEC,
         GIMTEC,
         SUBTEC) = fc[:, 14:17].T

    elif version == 1.0:
        # version 1.0: GIMTRACK, TOPTEC, SUBTEC
        (TOPTEC,
         GIMTEC,
         SUBTEC) = fc[:, 12:15].T

    # save for external plotting
    date_str = re.findall(r'\d{8}', os.path.basename(tec_file))[0]
    date_obj = dt.datetime.strptime(date_str, '%Y%m%d')
    tDict = {}
    tDict['date'] = date_str
    tDict['time'] = date_obj
    tDict['lat'] = lat
    tDict['lon'] = lon
    tDict['tec_ipp']     = tec_ipp
    tDict['tec_top_tpp'] = tec_top_tpp
    tDict['tec_sub_ipp'] = tec_sub_ipp
    tDict['tec_sub_tpp'] = tec_sub_tpp

    return tDict


def check_date_list_against_reference(date_list, dset_list, date_list_ref, fill_value=np.nan):
    """Check input date/dset_list against the reference date list:
    1. remove dates that are not existed in the reference date list
    2. fill data of the missing dates
    """
    # remove dates not in date_list_ref
    flag = np.array([i in date_list_ref for i in date_list], dtype=np.bool_)
    if np.sum(flag) < len(date_list):
        date_list = np.array(date_list)[flag].tolist()
        dset_list = np.array(dset_list)[flag].tolist()

    # fill missing dates from date_list_ref
    flag = np.array([i in date_list for i in date_list_ref], dtype=np.bool_)
    if np.sum(flag) < len(date_list_ref):
        date_list = list(date_list_ref)
        dset_list_bk = list(dset_list)
        dset_list = np.zeros(len(date_list_ref), dtype=np.float32) * fill_value
        dset_list[flag] = dset_list_bk

    return date_list, dset_list



#!/usr/bin/env python3
############################################################
# Program is part of MintPy                                #
# Copyright (c) 2013, Zhang Yunjun, Heresh Fattahi         #
# Author: Antonio Valentino, Zhang Yunjun, Aug 2022        #
############################################################


import os
import sys

from mintpy.utils import arg_utils

###########################################################################################
EXAMPLE = """example:
  tsview.py timeseries.h5
  tsview.py timeseries.h5  --wrap
  tsview.py timeseries.h5  --yx 300 400 --zero-first  --nodisplay
  tsview.py geo_timeseries.h5  --lalo 33.250 131.665  --nodisplay
  tsview.py slcStack.h5 -u dB -v 20 60 -c gray

  # press left / right key to slide images

  # multiple time-series files
  tsview.py timeseries_ERA5_ramp_demErr.h5 timeseries_ERA5_ramp.h5 timeseries_ERA5.h5 timeseries.h5 --off 5
  tsview.py timeseries_ERA5_ramp_demErr.h5 ../GIANT/Stack/LS-PARAMS.h5 --off 5 --label mintpy giant
"""


def create_parser(subparsers=None):
    synopsis = 'Interactive time-series viewer'
    epilog = EXAMPLE
    name = __name__.split('.')[-1]
    parser = arg_utils.create_argument_parser(
        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)

    parser.add_argument('file', nargs='+',
                        help='time-series file to display\n'
                             'i.e.: timeseries_ERA5_ramp_demErr.h5 (MintPy)\n'
                             '      LS-PARAMS.h5 (GIAnT)\n'
                             '      S1_IW12_128_0593_0597_20141213_20180619.he5 (HDF-EOS5)')
    parser.add_argument('--label', dest='file_label', nargs='*',
                        help='labels to display for multiple input files')
    parser.add_argument('--ylim', dest='ylim', nargs=2, metavar=('YMIN', 'YMAX'), type=float,
                        help='Y limits for point plotting.')
    parser.add_argument('--tick-right', dest='tick_right', action='store_true',
                        help='set tick and tick label to the right')
    parser.add_argument('-l','--lookup', dest='lookup_file', type=str,
                        help='lookup table file')
    parser.add_argument('--no-show-img','--not-show-image', dest='disp_fig_img', action='store_false',
                        help='do NOT show the map figure.\n'
                             'Useful for plotting a point time series only.\n'
                             'This option requires --yx/lalo input.')

    parser.add_argument('-n', dest='idx', metavar='NUM', type=int,
                        help='Epoch/slice number for initial display.')
    parser.add_argument('--error', dest='error_file',
                        help='txt file with error for each date.')

    # time info
    parser.add_argument('--start-date', dest='start_date', type=str,
                        help='start date of displacement to display')
    parser.add_argument('--end-date', dest='end_date', type=str,
                        help='end date of displacement to display')
    parser.add_argument('--exclude', '--ex', dest='ex_date_list', nargs='*', default=['exclude_date.txt'],
                        help='Exclude date shown as gray.')
    parser.add_argument('--zf', '--zero-first', dest='zero_first', action='store_true',
                        help='Set displacement at first acquisition to zero.')
    parser.add_argument('--off','--offset', dest='offset', type=float,
                        help='Offset for each timeseries file.')

    parser.add_argument('--noverbose', dest='print_msg', action='store_false',
                        help='Disable the verbose message printing.')

    # temporal model fitting
    parser.add_argument('--nomodel', '--nofit', dest='plot_model', action='store_false',
                        help='Do not plot the prediction of the time function (deformation model) fitting.')
    parser.add_argument('--plot-model-conf-int', '--plot-fit-conf-int', dest='plot_model_conf_int', action='store_true',
                        help='Plot the time function prediction confidence intervals.\n'
                             '[!-- Preliminary feature alert! --!]\n'
                             '[!-- This feature is NOT throughly checked. '
                             'Read the code before use. Interpret at your own risk! --!]')

    parser = arg_utils.add_timefunc_argument(parser)

    # pixel of interest
    pixel = parser.add_argument_group('Pixel Input')
    pixel.add_argument('--yx', type=int, metavar=('Y', 'X'), nargs=2,
                       help='initial pixel to plot in Y/X coord')
    pixel.add_argument('--lalo', type=float, metavar=('LAT', 'LON'), nargs=2,
                       help='initial pixel to plot in lat/lon coord')

    pixel.add_argument('--marker', type=str, default='o',
                       help='marker style (default: %(default)s).')
    pixel.add_argument('--ms', '--markersize', dest='marker_size', type=float, default=6.0,
                       help='marker size (default: %(default)s).')
    pixel.add_argument('--lw', '--linewidth', dest='linewidth', type=float, default=0,
                       help='line width (default: %(default)s).')
    pixel.add_argument('--ew', '--edgewidth', dest='edge_width', type=float, default=1.0,
                       help='Edge width for the error bar (default: %(default)s)')

    # other groups
    parser = arg_utils.add_data_disp_argument(parser)
    parser = arg_utils.add_dem_argument(parser)
    parser = arg_utils.add_figure_argument(parser)
    parser = arg_utils.add_gps_argument(parser)
    parser = arg_utils.add_mask_argument(parser)
    parser = arg_utils.add_map_argument(parser)
    parser = arg_utils.add_memory_argument(parser)
    parser = arg_utils.add_reference_argument(parser)
    parser = arg_utils.add_save_argument(parser)
    parser = arg_utils.add_subset_argument(parser)

    return parser


def cmd_line_parse(iargs=None):
    # parse
    parser = create_parser()
    inps = parser.parse_args(args=iargs)

    # save argv (to check the manually specified arguments)
    # use iargs        for python call
    # use sys.argv[1:] for command line call
    inps.argv = iargs if iargs else sys.argv[1:]

    # check: --gps-comp option (not implemented for tsview yet)
    if inps.gps_component:
        msg = f'--gps-comp is not supported for {os.path.basename(__file__)}'
        raise NotImplementedError(msg)

    # check: --label option (same number as input files)
    if inps.file_label:
        if len(inps.file_label) != len(inps.file):
            raise Exception('input number of labels != number of files.')

    # check: coupled options
    if not inps.save_fig and (inps.outfile or not inps.disp_fig):
        inps.save_fig = True

    if inps.flip_lr or inps.flip_ud:
        inps.auto_flip = False

    if inps.ylim:
        inps.ylim = sorted(inps.ylim)

    if inps.zero_mask:
        inps.mask_file = 'no'

    if not inps.disp_fig_img:
        if not inps.yx and not inps.lalo:
            inps.disp_fig_img = True
            msg = 'WARNING: --yx/lalo is required for --no-show-img but NOT found! '
            msg += 'Ignore it and continue'
            print(msg)

    # check: --noverbose option
    # print tsview.py command line if --noverbose
    if not inps.print_msg:
        print('tsview.py', ' '.join(inps.argv))

    # default: -u / -c / --fig-size options
    inps.disp_unit = inps.disp_unit if inps.disp_unit else 'cm'
    inps.colormap = inps.colormap if inps.colormap else 'jet'
    inps.fig_size = inps.fig_size if inps.fig_size else [8.0, 4.5]

    return inps


###########################################################################################
def main(iargs=None):
    # parse
    inps = cmd_line_parse(iargs)

    # import
    from mintpy.tsview import timeseriesViewer

    # run
    obj = timeseriesViewer(iargs=iargs)
    obj.configure(inps)
    obj.plot()
    #obj.fig_img.canvas.mpl_disconnect(obj.cid)


#########################################################################################
if __name__ == '__main__':
    main(sys.argv[1:])
